# populator/helpers/mdraid.py
# MD RAID backend code for populating a DeviceTree.
#
# Copyright (C) 2009-2015  Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# the GNU Lesser General Public License v.2, or (at your option) any later
# version. This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY expressed or implied, including the implied
# warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
# the GNU Lesser General Public License for more details.  You should have
# received a copy of the GNU Lesser General Public License along with this
# program; if not, write to the Free Software Foundation, Inc., 51 Franklin
# Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat trademarks
# that are incorporated in the source code or documentation are not subject
# to the GNU Lesser General Public License and may only be used or
# replicated with the express permission of Red Hat, Inc.
#
# Red Hat Author(s): David Lehman <dlehman@redhat.com>
#

import gi
gi.require_version("BlockDev", "3.0")

from gi.repository import BlockDev as blockdev

import re

from ... import udev
from ...devicelibs import raid
from ...devices import MDRaidArrayDevice, MDContainerDevice
from ...devices import device_path_to_name
from ...errors import DeviceError, NoParentsError
from ...flags import flags
from ...storage_log import log_method_call
from .devicepopulator import DevicePopulator
from .formatpopulator import FormatPopulator

import logging
log = logging.getLogger("blivet")


class MDDevicePopulator(DevicePopulator):
    @classmethod
    def match(cls, data):
        return (udev.device_is_md(data) and
                not udev.device_get_md_container(data))

    def run(self):
        name = udev.device_get_md_name(self.data)
        log_method_call(self, name=name)

        try:
            self._devicetree._add_parent_devices(self.data)
        except NoParentsError:
            log.error("no parents found for mdarray %s, skipping", name)
            return None

        # try to get the device again now that we've got all the parents
        device = self._devicetree.get_device_by_name(name, incomplete=flags.allow_imperfect_devices)

        if device is None:
            try:
                uuid = udev.device_get_md_uuid(self.data)
            except KeyError:
                log.warning("failed to obtain uuid for mdraid device")
            else:
                device = self._devicetree.get_device_by_uuid(uuid, incomplete=flags.allow_imperfect_devices)

        if device and name:
            # update the device instance with the real name in case we had to
            # look it up by something other than name
            device.name = name

        if device is None:
            # if we get here, we found all of the parent devices and
            # something must be wrong -- if all of the parents are in
            # the tree, this device should be as well
            if name is None:
                name = udev.device_get_name(self.data)
                path = "/dev/" + name
            else:
                path = "/dev/md/" + name

            log.error("failed to scan md array %s", name)
            try:
                blockdev.md.deactivate(path)
            except blockdev.MDRaidError:
                log.error("failed to stop broken md array %s", name)

        return device


class MDFormatPopulator(FormatPopulator):
    priority = 100
    _type_specifier = "mdmember"

    def _get_kwargs(self):
        kwargs = super(MDFormatPopulator, self)._get_kwargs()
        kwargs["biosraid"] = udev.device_is_biosraid_member(self.data)
        if not kwargs["biosraid"]:
            try:
                # ID_FS_UUID contains the array UUID
                kwargs["md_uuid"] = udev.device_get_uuid(self.data)
            except KeyError:
                log.warning("mdraid member %s has no md uuid", udev.device_get_name(self.data))
        else:
            # for BIOS RAIDs we can't get the UUID from udev, we'll get it from mdadm in `run` below
            kwargs["md_uuid"] = None

        # reset the uuid to the member-specific value
        # this will be None for members of v0 metadata arrays
        kwargs["uuid"] = udev.device_get_md_device_uuid(self.data)

        return kwargs

    def run(self):
        super(MDFormatPopulator, self).run()
        try:
            md_info = blockdev.md.examine(self.device.path)
        except blockdev.MDRaidError as e:
            # This could just mean the member is not part of any array.
            log.debug("blockdev.md.examine error: %s", str(e))
            return

        # Use mdadm info if udev info is missing
        md_uuid = md_info.uuid
        self.device.format.md_uuid = self.device.format.md_uuid or md_uuid
        md_array = self._devicetree.get_device_by_uuid(self.device.format.md_uuid, incomplete=True)

        if md_array:
            md_array.parents.append(self.device)
        else:
            # create the array with just this one member
            # level is reported as, eg: "raid1"
            md_level = md_info.level
            md_devices = md_info.num_devices

            if md_level is None:
                log.warning("invalid data for %s: no RAID level", self.device.name)
                return

            # md_examine yields metadata (MD_METADATA) only for metadata version > 0.90
            # if MD_METADATA is missing, assume metadata version is 0.90
            md_metadata = md_info.metadata or "0.90"
            md_name = None

            # check the list of devices udev knows about to see if the array
            # this device belongs to is already active
            # XXX This is mainly for containers now since their name/device is
            #     not given by mdadm examine as we run it.
            for dev in udev.get_devices():
                if not udev.device_is_md(dev):
                    continue

                try:
                    dev_uuid = udev.device_get_md_uuid(dev)
                    dev_level = udev.device_get_md_level(dev)
                except KeyError:
                    continue

                if dev_uuid is None or dev_level is None:
                    continue

                if dev_uuid == md_uuid and dev_level == md_level:
                    md_name = udev.device_get_md_name(dev)
                    break

            md_path = md_info.device or ""
            if md_path and not md_name:
                md_name = device_path_to_name(md_path)
                if re.match(r'md\d+$', md_name):
                    # md0 -> 0
                    md_name = md_name[2:]

                if md_name:
                    array = self._devicetree.get_device_by_name(md_name, incomplete=True)
                    if array and array.uuid != md_uuid:
                        log.error("found multiple devices with the name %s", md_name)

            if md_name:
                log.info("using name %s for md array containing member %s",
                         md_name, self.device.name)
            else:
                log.error("failed to determine name for the md array %s", (md_uuid or "unknown"))
                return

            array_type = MDRaidArrayDevice
            try:
                if raid.get_raid_level(md_level) is raid.Container and \
                   getattr(self.device.format, "biosraid", False):
                    array_type = MDContainerDevice
            except raid.RaidError as e:
                log.error("failed to create md array: %s", e)
                return

            try:
                md_array = array_type(
                    md_name,
                    level=md_level,
                    member_devices=md_devices,
                    uuid=md_uuid,
                    metadata_version=md_metadata,
                    exists=True
                )
            except (ValueError, DeviceError) as e:
                log.error("failed to create md array: %s", e)
                return

            md_array.update_sysfs_path()
            md_array.parents.append(self.device)
            self._devicetree._add_device(md_array)
            if md_array.status:
                array_info = udev.get_device(md_array.sysfs_path)
                if not array_info:
                    log.error("failed to get udev data for %s", md_array.name)
                    return

                self._devicetree.handle_device(array_info, update_orig_fmt=True)

    def update(self):
        # update array based on current md data
        pass



# stratis.py
# Device format classes for anaconda's storage configuration module.
#
# Copyright (C) 2020  Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# the GNU General Public License v.2, or (at your option) any later version.
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY expressed or implied, including the implied warranties of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.  You should have received a copy of the
# GNU General Public License along with this program; if not, write to the
# Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.  Any Red Hat trademarks that are incorporated in the
# source code or documentation are not subject to the GNU General Public
# License and may only be used or replicated with the express permission of
# Red Hat, Inc.
#
# Red Hat Author(s): Vojtech Trefny <vtrefny@redhat.com>
#

import os

from ..storage_log import log_method_call
from ..i18n import N_
from ..size import Size
from ..errors import StratisError
from ..devicelibs import stratis
from . import DeviceFormat, register_device_format

import logging
log = logging.getLogger("blivet")


class StratisBlockdev(DeviceFormat):
    """ A Stratis block device """

    _type = "stratis"
    _name = N_("Stratis block device")
    _udev_types = ["stratis"]
    _formattable = True                  # can be formatted
    _supported = True                    # is supported
    _linux_native = True                 # for clearpart
    _min_size = Size("1 GiB")
    _packages = ["stratisd"]             # required packages
    _resizable = False

    def __init__(self, **kwargs):
        """
            :keyword device: path to the block device node
            :keyword uuid: this Stratis block device UUID (not the pool UUID)
            :keyword exists: indicates whether this is an existing format
            :type exists: bool
            :keyword pool_name: the name of the pool this block device belongs to
            :keyword pool_uuid: the UUID of the pool this block device belongs to
            :keyword locked_pool: whether this block device belongs to a locked pool or not
            :type locked_pool: bool
            :keyword locked_pool_key_desc: kernel keyring description for locked pool
            :type locked_pool_key_desc: str
            :keyword passphrase: passphrase for the locked pool
            :type passphrase: str
            :keyword key_file: path to a file containing a key
            :type key_file: str

            .. note::

                The 'device' kwarg is required for existing formats. For non-
                existent formats, it is only necessary that the :attr:`device`
                attribute be set before the :meth:`create` method runs. Note
                that you can specify the device at the last moment by specifying
                it via the 'device' kwarg to the :meth:`create` method.
        """
        log_method_call(self, **kwargs)
        DeviceFormat.__init__(self, **kwargs)

        self.pool_name = kwargs.get("pool_name")
        self.pool_uuid = kwargs.get("pool_uuid")
        self.locked_pool = kwargs.get("locked_pool")
        self.locked_pool_key_desc = kwargs.get("locked_pool_key_desc")

        self.__passphrase = kwargs.get("passphrase")
        self._key_file = kwargs.get("key_file")

    def __repr__(self):
        s = DeviceFormat.__repr__(self)
        s += ("  pool_name = %(pool_name)s  pool_uuid = %(pool_uuid)s  locked_pool = %(locked_pool)s" %
              {"pool_name": self.pool_name, "pool_uuid": self.pool_uuid,
               "locked_pool": self.locked_pool})
        return s

    @property
    def status(self):
        return self.exists and (self.pool_name or self.pool_uuid)

    @property
    def dict(self):
        d = super(StratisBlockdev, self).dict
        d.update({"pool_name": self.pool_name, "pool_uuid": self.pool_uuid})
        return d

    @property
    def key_file(self):
        """ Path to key file to be used in /etc/crypttab """
        return self._key_file

    def _set_passphrase(self, passphrase):
        """ Set the passphrase used to access this device. """
        self.__passphrase = passphrase

    passphrase = property(fset=_set_passphrase)

    @property
    def has_key(self):
        return bool((self.__passphrase not in ["", None]) or
                    (self._key_file and os.access(self._key_file, os.R_OK)))

    def unlock_pool(self):
        if not self.locked_pool:
            raise StratisError("This device doesn't contain a locked Stratis pool")

        if not self.has_key:
            raise StratisError("No passphrase/key file for the locked Stratis pool")

        if not self.locked_pool_key_desc:
            raise StratisError("No key description for the locked Stratis pool")

        stratis.set_key(self.locked_pool_key_desc, self.__passphrase, self.key_file)
        stratis.unlock_pool(self.pool_uuid)


register_device_format(StratisBlockdev)



import os

import blivet
from blivet.size import Size
from blivet.util import set_up_logging, create_sparse_tempfile
from blivet.devices import LUKSDevice

set_up_logging()
b = blivet.Blivet()   # create an instance of Blivet (don't add system devices)

# create a disk image file on which to create new devices
disk1_file = create_sparse_tempfile("disk1", Size("100GiB"))
b.disk_images["disk1"] = disk1_file

b.reset()

try:
    disk1 = b.devicetree.get_device_by_name("disk1")

    b.initialize_disk(disk1)

    pv = b.new_partition(size=Size("50GiB"), fmt_type="lvmpv")
    b.create_device(pv)

    # allocate the partitions (decide where and on which disks they'll reside)
    blivet.partitioning.do_partitioning(b)

    vg = b.new_vg(parents=[pv])
    b.create_device(vg)

    # new lv with base size 5GiB and unbounded growth and an ext4 filesystem
    dev = b.new_lv(fmt_type="ext4", size=Size("5GiB"), grow=True,
                   parents=[vg], name="unbounded")
    b.create_device(dev)

    # new lv with base size 5GiB and growth up to 15GiB and an ext4 filesystem
    dev = b.new_lv(fmt_type="ext4", size=Size("5GiB"), grow=True,
                   maxsize=Size("15GiB"), parents=[vg], name="bounded")
    b.create_device(dev)

    # new lv with a fixed size of 2GiB formatted as swap space
    dev = b.new_lv(fmt_type="swap", size=Size("2GiB"), parents=[vg])
    b.create_device(dev)

    # new LUKS encrypted lv with fixed size of 10GiB and ext4 filesystem
    dev = b.new_lv(fmt_type="luks", fmt_args={"passphrase": "12345"},
                   size=Size("10GiB"), parents=[vg], name="encrypted")
    b.create_device(dev)

    luks_dev = LUKSDevice(name="luks-%s" % dev.name,
                          size=dev.size, parents=[dev])
    b.create_device(luks_dev)

    luks_fmt = blivet.formats.get_format(fmt_type="ext4", device=luks_dev.path)
    b.format_device(luks_dev, luks_fmt)

    # allocate the growable lvs
    blivet.partitioning.grow_lvm(b)
    print(b.devicetree)

    # write the new partitions to disk and format them as specified
    b.do_it()
    print(b.devicetree)
finally:
    b.devicetree.teardown_disk_images()
    os.unlink(disk1_file)



import os

import blivet
from uuid import UUID
from blivet.devices import LUKSDevice
from blivet.size import Size
from blivet.flags import flags
from blivet import formats
from blivet.formats import disklabel
from blivet.util import set_up_logging, create_sparse_tempfile

set_up_logging()
b = blivet.Blivet()   # create an instance of Blivet (don't add system devices)

# create two disk image files on which to create new devices
disk1_file = create_sparse_tempfile("disk1", Size("100GiB"))
b.disk_images["disk1"] = disk1_file
disk2_file = create_sparse_tempfile("disk2", Size("100GiB"))
b.disk_images["disk2"] = disk2_file

b.reset()

disklabel.DiskLabel.set_default_label_type("gpt")
flags.gpt_discoverable_partitions = True

try:
    disk1 = b.devicetree.get_device_by_name("disk1")
    disk2 = b.devicetree.get_device_by_name("disk2")

    b.initialize_disk(disk1)
    b.initialize_disk(disk2)

    # new partition on either disk1 or disk2 with base size 10GiB and growth
    # up to a maximum size of 50GiB
    root = b.new_partition(size=Size("10MiB"), maxsize=Size("50GiB"),
                           grow=True, parents=[disk1, disk2],
                           mountpoint="/")
    b.create_device(root)

    # new partition on disk1 with base size 5GiB and unbounded growth and an
    # ext4 filesystem
    home = b.new_partition(fmt_type="ext4", size=Size("5GiB"), grow=True,
                           parents=[disk1], mountpoint="/home")
    b.create_device(home)

    var = b.new_partition(fmt_type="luks",
                          fmt_args={
                              "passphrase": "123456",
                          }, size=Size("200MiB"),
                          parents=[disk2],
                          mountpoint="/var")
    b.create_device(var)

    varenc = LUKSDevice(
        name="luks-user", size=var.size, parents=var)
    b.create_device(varenc)

    varfs = formats.get_format(
        fmt_type="ext4", device=varenc.path, mountpoint="/usr")
    b.format_device(varenc, varfs)

    # new partition on any suitable disk with a fixed size of 2GiB formatted
    # as swap space
    swap = b.new_partition(fmt_type="swap", size=Size("2GiB"))
    b.create_device(swap)

    # allocate the partitions (decide where and on which disks they'll reside)
    blivet.partitioning.do_partitioning(b)
    print(b.devicetree)

    # write the new partitions to disk and format them as specified
    b.do_it()
    print(b.devicetree)

    print("/ assigned GUID %s" % UUID(bytes=root.parted_partition.type_uuid))
    print("/home assigned GUID %s" % UUID(bytes=home.parted_partition.type_uuid))
    print("/var assigned GUID %s" % UUID(bytes=var.parted_partition.type_uuid))
    print("<swap> assigned GUID %s" % UUID(bytes=swap.parted_partition.type_uuid))

finally:
    b.devicetree.teardown_disk_images()
    os.unlink(disk1_file)
    os.unlink(disk2_file)



import blivet
from blivet.util import set_up_logging

set_up_logging()
b = blivet.Blivet()   # create an instance of Blivet
b.reset()             # detect system storage configuration

print(b.devicetree)



#
# Copyright (C) 2016  Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# the GNU General Public License v.2, or (at your option) any later version.
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY expressed or implied, including the implied warranties of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.  You should have received a copy of the
# GNU General Public License along with this program; if not, write to the
# Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.  Any Red Hat trademarks that are incorporated in the
# source code or documentation are not subject to the GNU General Public
# License and may only be used or replicated with the express permission of
# Red Hat, Inc.
#
# Red Hat Author(s): David Lehman <dlehman@redhat.com>
#
import sys

import dbus

from .. import Blivet
from ..callbacks import callbacks
from ..devicefactory import DEVICE_TYPE_PARTITION, DEVICE_TYPE_LVM, DEVICE_TYPE_LVM_THINP
from ..devicefactory import DEVICE_TYPE_MD, DEVICE_TYPE_BTRFS
from ..errors import StorageError
from ..size import Size
from ..util import ObjectID
from .action import DBusAction
from .constants import BLIVET_INTERFACE, BLIVET_OBJECT_PATH, BUS_NAME
from .device import DBusDevice
from .format import DBusFormat
from .object import DBusObject


def sorted_object_paths_from_list(obj_list):
    objects = sorted(obj_list, key=lambda o: o.id)
    return [o.object_path for o in objects]


class DBusBlivet(DBusObject):
    """ This class provides the main entry point to the Blivet1 service.

        It provides methods for controlling the blivet service and querying its
        state.
    """
    transient = False

    def __init__(self, manager):
        super(DBusBlivet, self).__init__(manager)
        self._blivet = Blivet()
        self._id = ObjectID().id
        self._manager.add_object(self)
        self._set_up_callbacks()

    def _set_up_callbacks(self):
        callbacks.device_added.add(self._device_added)
        callbacks.device_removed.add(self._device_removed)
        callbacks.format_added.add(self._format_added)
        callbacks.format_removed.add(self._format_removed)
        callbacks.action_added.add(self._action_added)
        callbacks.action_removed.add(self._action_removed)
        callbacks.action_executed.add(self._action_executed)

    @property
    def id(self):
        return self._id

    @property
    def object_path(self):
        return BLIVET_OBJECT_PATH

    @property
    def interface(self):
        return BLIVET_INTERFACE

    @property
    def properties(self):
        props = {"Devices": self.ListDevices(),
                 "DEVICE_TYPE_LVM": DEVICE_TYPE_LVM,
                 "DEVICE_TYPE_LVM_THINP": DEVICE_TYPE_LVM_THINP,
                 "DEVICE_TYPE_PARTITION": DEVICE_TYPE_PARTITION,
                 "DEVICE_TYPE_MD": DEVICE_TYPE_MD,
                 "DEVICE_TYPE_BTRFS": DEVICE_TYPE_BTRFS}
        return props

    def _device_removed(self, device, keep=True):
        """ Update ObjectManager interface after a device is removed. """
        # Make sure the format gets removed in case the device was removed w/o
        # removing the format first.
        removed_fmt = self._manager.get_object_by_id(device.format.id)
        if removed_fmt and removed_fmt.present:
            self._format_removed(device, device.format, keep=keep)
        elif removed_fmt and not keep:
            self._format_removed(device, device.format, keep=False)

        removed = self._manager.get_object_by_id(device.id)
        self._manager.remove_object(removed)
        if keep:
            removed.present = False
            self._manager.add_object(removed)

    def _device_added(self, device):
        """ Update ObjectManager interface after a device is added. """
        added = self._manager.get_object_by_id(device.id)
        if added:
            # This device was previously removed. Restore it.
            added.present = True
        else:
            added = DBusDevice(device, self._manager)

        self._manager.add_object(added)

    def _format_removed(self, device, fmt, keep=True):  # pylint: disable=unused-argument
        removed = self._manager.get_object_by_id(fmt.id)
        if removed is None:
            return

        # We have to remove the object either way since its path will change.
        self._manager.remove_object(removed)
        if keep:
            removed.present = False
            self._manager.add_object(removed)

    def _format_added(self, device, fmt):  # pylint: disable=unused-argument
        added = self._manager.get_object_by_id(fmt.id)
        if added:
            # This format was previously removed. Restore it.
            added.present = True
        else:
            added = DBusFormat(fmt, self._manager)

        self._manager.add_object(added)

    def _action_removed(self, action):
        removed = self._manager.get_object_by_id(action.id)
        self._manager.remove_object(removed)

    def _action_added(self, action):
        added = DBusAction(action, self._manager)
        self._manager.add_object(added)

    def _action_executed(self, action):
        self._action_removed(action)

    def _list_dbus_devices(self, removed=False):
        dbus_devices = (d for d in self._manager.objects if isinstance(d, DBusDevice))
        return [d for d in dbus_devices if removed or d.present]

    def _get_device_by_object_path(self, object_path, removed=False):
        """ Return the StorageDevice corresponding to an object path. """
        dbus_device = self._manager.get_object_by_path(object_path)
        if dbus_device is None or not isinstance(dbus_device, DBusDevice):
            raise dbus.exceptions.DBusException('%s.DeviceNotFound' % BUS_NAME,
                                                'No device found with object path "%s".'
                                                % object_path)

        if not dbus_device.present and not removed:
            raise dbus.exceptions.DBusException('%s.DeviceNotFound' % BUS_NAME,
                                                'Device with object path "%s" has already been '
                                                'removed.' % object_path)

        return dbus_device._device

    @dbus.service.method(dbus_interface=BLIVET_INTERFACE)
    def Reset(self):
        """ Reset the Blivet instance and populate the device tree. """
        for obj in self._manager.objects:
            if obj.transient:
                self._manager.remove_object(obj)

        self._blivet.reset()

    @dbus.service.method(dbus_interface=BLIVET_INTERFACE)
    def Exit(self):
        """ Stop the blivet service. """
        sys.exit(0)

    @dbus.service.method(dbus_interface=BLIVET_INTERFACE, out_signature='ao')
    def ListDevices(self):
        """ Return a list of strings describing the devices in this system. """
        object_paths = sorted_object_paths_from_list(self._list_dbus_devices())
        return dbus.Array(object_paths, signature='o')

    @dbus.service.method(dbus_interface=BLIVET_INTERFACE, in_signature='s', out_signature='o')
    def ResolveDevice(self, spec):
        """ Return a string describing the device the given specifier resolves to. """
        device = self._blivet.devicetree.resolve_device(spec)
        if device is None:
            raise dbus.exceptions.DBusException('%s.DeviceLookupFailed' % BUS_NAME,
                                                'No device was found that matches the device '
                                                'descriptor "%s".' % spec)

        return self._manager.get_object_by_id(device.id).object_path

    @dbus.service.method(dbus_interface=BLIVET_INTERFACE, in_signature='o')
    def RemoveDevice(self, object_path):
        """ Remove a device and all devices built on it. """
        device = self._get_device_by_object_path(object_path)
        self._blivet.devicetree.recursive_remove(device)

    @dbus.service.method(dbus_interface=BLIVET_INTERFACE, in_signature='o')
    def InitializeDisk(self, object_path):
        """ Clear a disk and create a disklabel on it. """
        self.RemoveDevice(object_path)
        device = self._get_device_by_object_path(object_path)
        self._blivet.initialize_disk(device)

    @dbus.service.method(dbus_interface=BLIVET_INTERFACE)
    def Commit(self):
        """ Commit pending changes to disk. """
        try:
            self._blivet.do_it()
        except StorageError as e:
            raise dbus.exceptions.DBusException('%s.%s' % (BUS_NAME, e.__class__.__name__),
                                                "An error occured while committing the "
                                                "changes to disk: %s" % str(e))

    @dbus.service.method(dbus_interface=BLIVET_INTERFACE, in_signature='a{sv}', out_signature='o')
    def Factory(self, kwargs):
        disks = [self._get_device_by_object_path(p) for p in kwargs.pop("disks", [])]
        kwargs["disks"] = disks

        dbus_device = kwargs.pop("device", None)
        if dbus_device:
            device = self._get_device_by_object_path(dbus_device)
            kwargs["device"] = device

        size = kwargs.pop("size", None)
        if size is not None:
            kwargs["size"] = Size(size)

        try:
            device = self._blivet.factory_device(**kwargs)
        except StorageError as e:
            raise dbus.exceptions.DBusException('%s.%s' % (BUS_NAME, e.__class__.__name__),
                                                "An error occured while configuring the "
                                                "device: %s" % str(e))

        if device is None:
            object_path = '/'
        else:
            object_path = self._manager.get_object_by_id(device.id).object_path

        return object_path



import unittest

try:
    from unittest.mock import patch, Mock
except ImportError:
    from mock import patch, Mock

import blivet

from blivet.devices import PartitionDevice, DiskDevice, StorageDevice


class SuggestNameTestCase(unittest.TestCase):

    @patch("blivet.formats.fs.Ext4FS.supported", return_value=True)
    @patch("blivet.formats.fs.Ext4FS.formattable", return_value=True)
    def setUp(self, *args):  # pylint: disable=unused-argument,arguments-differ
        self.b = blivet.Blivet()

    def test_suggest_container_name(self):

        with patch("blivet.devicetree.DeviceTree.names", []):
            name = self.b.suggest_container_name(prefix="blivet")
            self.assertEqual(name, "blivet")

        with patch("blivet.devicetree.DeviceTree.names", ["blivet"]):
            name = self.b.suggest_container_name(prefix="blivet")
            self.assertEqual(name, "blivet00")

        with patch("blivet.devicetree.DeviceTree.names", ["blivet"] + ["blivet%02d" % i for i in range(100)]):
            with self.assertRaises(RuntimeError):
                self.b.suggest_container_name(prefix="blivet")

    def test_suggest_device_name(self):
        with patch("blivet.devicetree.DeviceTree.names", []):
            name = self.b.suggest_device_name()
            self.assertEqual(name, "00")

            name = self.b.suggest_device_name(prefix="blivet")
            self.assertEqual(name, "blivet00")

            name = self.b.suggest_device_name(mountpoint="/")
            self.assertEqual(name, "root")

            name = self.b.suggest_device_name(prefix="blivet", mountpoint="/")
            self.assertEqual(name, "blivet_root")

            name = self.b.suggest_device_name(parent=blivet.devices.Device(name="parent"), mountpoint="/")
            self.assertEqual(name, "root")

        with patch("blivet.devicetree.DeviceTree.names", ["00"]):
            name = self.b.suggest_device_name()
            self.assertEqual(name, "01")

        with patch("blivet.devicetree.DeviceTree.names", ["parent-root"]):
            name = self.b.suggest_device_name(parent=blivet.devices.Device(name="parent"), mountpoint="/")
            self.assertEqual(name, "root00")


class SortDevicesTest(unittest.TestCase):

    @patch("blivet.formats.fs.Ext4FS.supported", return_value=True)
    @patch("blivet.formats.fs.Ext4FS.formattable", return_value=True)
    def setUp(self, *args):  # pylint: disable=unused-argument,arguments-differ
        self.b = blivet.Blivet()

    def test_sort_devices(self):

        disk = DiskDevice("sda", parents=[], size=blivet.size.Size("1 GiB"))
        self.b.devicetree._add_device(disk)

        for i in range(1, 12):
            part = PartitionDevice(name="sda%d" % i, parents=[disk])
            part.parents = [disk]
            part._parted_partition = Mock(number=i)
            self.b.devicetree._add_device(part)

        self.assertEqual([d.name for d in self.b.devices],
                         ["sda"] + ["sda%d" % i for i in range(1, 12)])

        # disk children should be also sorted
        self.assertEqual([d.name for d in disk.children],
                         ["sda%d" % i for i in range(1, 12)])

        # add some "extra" devices just to be sure the "non-partition" sort still works
        self.b.devicetree._add_device(StorageDevice("sdb", parents=[]))
        self.b.devicetree._add_device(StorageDevice("nvme0n1", parents=[]))
        self.b.devicetree._add_device(StorageDevice("10", parents=[]))

        self.assertEqual([d.name for d in self.b.devices],
                         ["10", "nvme0n1", "sda"] + ["sda%d" % i for i in range(1, 12)] + ["sdb"])



# devices/cache.py
#
# Copyright (C) 2015  Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# the GNU General Public License v.2, or (at your option) any later version.
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY expressed or implied, including the implied warranties of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.  You should have received a copy of the
# GNU General Public License along with this program; if not, write to the
# Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.  Any Red Hat trademarks that are incorporated in the
# source code or documentation are not subject to the GNU General Public
# License and may only be used or replicated with the express permission of
# Red Hat, Inc.
#
# Red Hat Author(s): Vratislav Podzimek <vpodzime@redhat.com>
#

"""Module providing common helper classes, functions and other things related to
cached devices (like bcache, LVM cache and whatever appears in the future).

"""

from six import add_metaclass
import abc


@add_metaclass(abc.ABCMeta)
class Cache(object):

    """Abstract base class for cache objects providing the cache-related
    functionality on cached devices. Instances of this class are not expected to
    be devices (both in what they represent as well as not instances of the
    :class:`~.devices.Device` class) since they just provide the cache-related
    functionality of cached devices and are not devices on their own.

    """

    @property
    @abc.abstractmethod
    def size(self):
        """Size of the cache"""

    @property
    @abc.abstractmethod
    def exists(self):
        """Whether the cache (device) exists or not"""

    @property
    @abc.abstractmethod
    def stats(self):
        """Statistics for the cache
        :rtype: :class:`CacheStats`
        """

    @property
    @abc.abstractmethod
    def backing_device_name(self):
        """Name of the backing (big/slow) device of the cache (if any)"""

    @property
    @abc.abstractmethod
    def cache_device_name(self):
        """Name of the cache (small/fast) device of the cache (if any)"""

    @abc.abstractmethod
    def detach(self):
        """Detach the cache
        :returns: identifier of the detached cache that can be later used for attaching it back

        """


@add_metaclass(abc.ABCMeta)
class CacheStats(object):

    """Abstract base class for common statistics of caches (cached
    devices). Inheriting classes are expected to add (cache-)type-specific
    attributes on top of the common set.

    """

    @property
    @abc.abstractmethod
    def block_size(self):
        """block size of the cache"""

    @property
    @abc.abstractmethod
    def size(self):
        """size of the cache"""

    @property
    @abc.abstractmethod
    def used(self):
        """how much of the cache is used"""

    @property
    @abc.abstractmethod
    def hits(self):
        """number of hits"""

    @property
    @abc.abstractmethod
    def misses(self):
        """number of misses"""


@add_metaclass(abc.ABCMeta)
class CacheRequest(object):

    """Abstract base class for cache requests specifying cache parameters for a
    cached device

    """
    @property
    @abc.abstractmethod
    def size(self):
        """Requested size"""

    @property
    @abc.abstractmethod
    def fast_devs(self):
        """Devices (type-specific) to allocate/create the cache on"""

    @property
    @abc.abstractmethod
    def mode(self):
        """Mode the cache should use"""



import os
import tempfile
import unittest

from blivet.formats.luks import LUKS, Integrity
from blivet.devicelibs import crypto
from blivet.errors import LUKSError
from blivet.size import Size

from . import loopbackedtestcase


class LUKSTestCase(loopbackedtestcase.LoopBackedTestCase):

    def __init__(self, methodName='run_test'):
        super(LUKSTestCase, self).__init__(methodName=methodName, device_spec=[Size("100 MiB")])
        self.fmt = LUKS(passphrase="password")

    def test_size(self):
        self.fmt.device = self.loop_devices[0]

        # create and open the luks format
        self.fmt.create()
        self.fmt.setup()
        self.addCleanup(self._luks_close)

        # without update_size_info size should be 0
        self.assertEqual(self.fmt.current_size, Size(0))

        # get current size
        self.fmt.update_size_info()
        self.assertGreater(self.fmt.current_size, Size(0))

    def test_resize(self):
        self.fmt.device = self.loop_devices[0]

        # create and open the luks format
        self.fmt.create()
        self.fmt.setup()
        self.addCleanup(self._luks_close)

        # get current size to make format resizable
        self.assertFalse(self.fmt.resizable)
        self.fmt.update_size_info()
        self.assertTrue(self.fmt.resizable)

        # resize the format
        new_size = Size("50 MiB")
        self.fmt.target_size = new_size
        self.fmt.do_resize()

        # get current size
        self.fmt.update_size_info()
        self.assertEqual(self.fmt.current_size, new_size)

    def test_map_name(self):
        self.fmt.device = self.loop_devices[0]

        # create and open the luks format
        self.fmt.create()
        self.fmt.setup()
        self.addCleanup(self._luks_close)

        self.assertEqual(self.fmt.map_name, "luks-%s" % self.fmt.uuid)
        self.assertTrue(self.fmt.status)

        self.fmt.teardown()
        self.assertFalse(self.fmt.status)

    def test_add_remove_passphrase(self):
        self.fmt.device = self.loop_devices[0]

        # create the luks format
        self.fmt.create()

        # add a new passphrase
        self.fmt.add_passphrase("password2")

        # we should now be able to setup the format using this passphrase
        self.fmt.passphrase = "password2"
        self.fmt.setup()
        self.fmt.teardown()

        # remove the original passphrase
        self.fmt.passphrase = "password"
        self.fmt.remove_passphrase()

        # now setup should fail
        with self.assertRaises(LUKSError):
            self.fmt.setup()

        # setup with the new passphrase should still be possible
        self.fmt.passphrase = "password2"
        self.fmt.setup()
        self.fmt.teardown()

    def test_setup_keyfile(self):
        self.fmt.device = self.loop_devices[0]

        with tempfile.NamedTemporaryFile(prefix="blivet_test") as temp:
            temp.write(b"password2")

            # create the luks format with both passphrase and keyfile
            self.fmt._key_file = temp.name
            self.fmt.create()

            # open first with just password
            self.fmt._key_file = None
            self.fmt.setup()
            self.fmt.teardown()

            # now with keyfile
            self.fmt._key_file = temp.name
            self.fmt.passphrase = None
            self.fmt.setup()
            self.fmt.teardown()

    def _luks_close(self):
        self.fmt.teardown()


@unittest.skipUnless(Integrity._plugin.available, "Integrity support not available")
class IntegrityTestCase(loopbackedtestcase.LoopBackedTestCase):

    def __init__(self, methodName='run_test'):
        super(IntegrityTestCase, self).__init__(methodName=methodName, device_spec=[Size("100 MiB")])

    def test_integrity(self):
        fmt = Integrity(device=self.loop_devices[0])
        self.assertEqual(fmt.algorithm, crypto.DEFAULT_INTEGRITY_ALGORITHM)

        # create and open the integrity format
        fmt.create()
        fmt.setup()
        self.assertTrue(fmt.status)
        self.assertTrue(os.path.exists("/dev/mapper/%s" % fmt.map_name))

        fmt.teardown()
        self.assertFalse(fmt.status)
        self.assertFalse(os.path.exists("/dev/mapper/%s" % fmt.map_name))



try:
    from unittest.mock import patch
except ImportError:
    from mock import patch

from contextlib import contextmanager

import unittest

from blivet.formats.lvmpv import LVMPhysicalVolume


class LVMPVNodevTestCase(unittest.TestCase):

    @contextmanager
    def patches(self):
        patchers = dict()
        mocks = dict()

        patchers["blockdev"] = patch("blivet.formats.lvmpv.blockdev")
        patchers["lvm"] = patch("blivet.formats.lvmpv.lvm")
        patchers["vgs_info"] = patch("blivet.formats.lvmpv.vgs_info")
        patchers["os"] = patch("blivet.formats.lvmpv.os")

        for name, patcher in patchers.items():
            mocks[name] = patcher.start()

        yield mocks

        for patcher in patchers.values():
            patcher.stop()

    def test_lvm_devices(self):
        fmt = LVMPhysicalVolume(device="/dev/test")

        with self.patches() as mock:
            # LVM devices file not enabled/supported -> devices_add should not be called
            mock["lvm"].HAVE_LVMDEVICES = False

            fmt._create()

            mock["blockdev"].lvm.devices_add.assert_not_called()

        with self.patches() as mock:
            # LVM devices file enabled and devices file exists -> devices_add should be called
            mock["lvm"].HAVE_LVMDEVICES = True
            mock["os"].path.exists.return_value = True

            fmt._create()

            mock["blockdev"].lvm.devices_add.assert_called_with("/dev/test")

        with self.patches() as mock:
            # LVM devices file enabled and devices file doesn't exist
            # and no existing VGs present -> devices_add should be called
            mock["lvm"].HAVE_LVMDEVICES = True
            mock["os"].path.exists.return_value = False
            mock["vgs_info"].cache = {}

            fmt._create()

            mock["blockdev"].lvm.devices_add.assert_called_with("/dev/test")

        with self.patches() as mock:
            # LVM devices file enabled and devices file doesn't exist
            # and existing VGs present -> devices_add should not be called
            mock["lvm"].HAVE_LVMDEVICES = True
            mock["os"].path.exists.return_value = False
            mock["vgs_info"].cache = {"fake_vg_uuid": "fake_vg_data"}

            fmt._create()

            mock["blockdev"].lvm.devices_add.assert_not_called()



import unittest

from .vmbackedtestcase import VMBackedTestCase

from blivet.size import Size
from blivet import devicefactory
from blivet import util

from blivet.udev import trigger
from blivet.devices.lvm import LVMLogicalVolumeDevice


def recursive_getattr(x, attr, default=None):
    """ Resolve a possibly-dot-containing attribute name. """
    val = x
    for sub_attr in attr.split("."):
        try:
            val = getattr(val, sub_attr)
        except AttributeError:
            return default

    return val


class BlivetResetTestCase(VMBackedTestCase):

    """ A class to test the results of Blivet.reset (and DeviceTree.populate).

        Create a device stack on disk images, catalog a set of attributes of
        every device we created, reset the Blivet instance, and then verify
        that the devices are all discovered and have attributes matching those
        we cataloged previously.
    """
    _validate_attrs = ["name", "type", "size",
                       "format.type", "format.mountpoint", "format.label"]
    """ List of StorageDevice attributes to verify match across the reset. """

    _identifying_attrs = ["name", "type"]
    """ List of attributes that must match to identify a device. """

    def collect_expected_data(self):
        """ Collect the attribute data we plan to validate later. """
        # only look at devices on the disk images -- no loops, &c
        for device in (d for d in self.blivet.devices
                       if d.disks and
                       set(d.disks).issubset(set(self.blivet.disks))):
            attr_dict = {}
            device._parted_device = None  # force update from disk for size, &c
            for attr in self._validate_attrs:
                attr_dict[attr] = recursive_getattr(device, attr)

            self.device_attr_dicts.append(attr_dict)

    def setUp(self):
        super(BlivetResetTestCase, self).setUp()

        trigger(subsystem="block", action="change")

        self.device_attr_dicts = []
        self.collect_expected_data()

    def skip_attr(self, device, attr):
        """ Return True if attr should not be checked for device. """
        # pylint: disable=unused-argument
        return False

    def find_device(self, attr_dict):
        devices = self.blivet.devices  # + self.blivet.devicetree._hidden
        device = None
        for check in devices:
            match = all(attr_dict[attr] == recursive_getattr(check, attr)
                        for attr in self._identifying_attrs)
            if match:
                device = check
                break

        return device

    def test_run(self):
        """ Verify that the devices and their attributes match across reset. """
        #
        # populate the devicetree
        # XXX it would be better to test the results of a reboot
        #
        self.blivet.reset()

        for attr_dict in self.device_attr_dicts:
            #
            # verify we can find the device that corresponds to this attr_dict
            #
            device = self.find_device(attr_dict)
            device_id = "/".join(attr_dict[attr] for attr in self._identifying_attrs)
            self.assertIsNotNone(device, msg="failed to find %s" % device_id)

            #
            # verify that all attributes match across the reset
            #
            for attr, expected in iter(attr_dict.items()):
                if self.skip_attr(device, attr):
                    continue

                actual = recursive_getattr(device, attr)
                self.assertEqual(actual, expected,
                                 msg=("attribute mismatch for %s: %s (%s/%s)"
                                      % (device_id, attr, expected, actual)))


class LVMTestCase(BlivetResetTestCase):

    """ Test that the devicetree can populate with the product of autopart. """

    def _set_up_storage(self):
        # This isn't exactly autopart, but it should be plenty close to it.
        self.blivet.factory_device(devicefactory.DEVICE_TYPE_LVM,
                                   size=Size("500 MiB"),
                                   disks=self.blivet.disks[:],
                                   fstype="swap")
        self.blivet.factory_device(devicefactory.DEVICE_TYPE_LVM,
                                   disks=self.blivet.disks[:])


class LVMSnapShotTestCase(BlivetResetTestCase):

    def _set_up_storage(self):
        self.blivet.factory_device(devicefactory.DEVICE_TYPE_LVM,
                                   size=Size("1 GiB"),
                                   label="ROOT",
                                   disks=self.blivet.disks[:],
                                   container_name="blivet_test",
                                   container_size=devicefactory.SIZE_POLICY_MAX)

    def setUp(self):
        super(BlivetResetTestCase, self).setUp()  # pylint: disable=bad-super-call
        root = self.blivet.lvs[0]
        snap = LVMLogicalVolumeDevice("rootsnap1", parents=[root.vg], origin=root,
                                      size=Size("768MiB"))
        self.blivet.create_device(snap)
        self.blivet.do_it()

        self.device_attr_dicts = []
        self.collect_expected_data()


class LVMThinpTestCase(BlivetResetTestCase):

    def _set_up_storage(self):
        self.blivet.factory_device(devicefactory.DEVICE_TYPE_LVM,
                                   size=Size("500 MiB"),
                                   fstype="swap",
                                   disks=self.blivet.disks[:])
        self.blivet.factory_device(devicefactory.DEVICE_TYPE_LVM_THINP,
                                   label="ROOT",
                                   disks=self.blivet.disks[:])


class LVMThinSnapShotTestCase(LVMThinpTestCase):

    def _set_up_storage(self):
        self.blivet.factory_device(devicefactory.DEVICE_TYPE_LVM_THINP,
                                   size=Size("1 GiB"),
                                   label="ROOT",
                                   disks=self.blivet.disks[:])

    def setUp(self):
        super(BlivetResetTestCase, self).setUp()  # pylint: disable=bad-super-call

        root = self.blivet.thinlvs[0]
        snap = LVMLogicalVolumeDevice("rootsnap1", parents=[root.pool],
                                      origin=root, seg_type="thin")
        self.blivet.create_device(snap)
        self.blivet.do_it()

        self.device_attr_dicts = []
        self.collect_expected_data()


class LVMRaidTestCase(BlivetResetTestCase):

    def _set_up_storage(self):
        # This isn't exactly autopart, but it should be plenty close to it.
        self.blivet.factory_device(devicefactory.DEVICE_TYPE_LVM,
                                   size=Size("500 MiB"),
                                   disks=self.blivet.disks[:],
                                   container_size=devicefactory.SIZE_POLICY_MAX)

    def setUp(self):
        super(BlivetResetTestCase, self).setUp()  # pylint: disable=bad-super-call
        vg_name = self.blivet.vgs[0].name
        util.run_program(["lvcreate", "-n", "raid", "--type", "raid1",
                          "-L", "100%", vg_name])
        self.device_attr_dicts = []
        self.collect_expected_data()


class LVMVDOTestCase(BlivetResetTestCase):

    def _set_up_storage(self):
        if not devicefactory.is_supported_device_type(devicefactory.DEVICE_TYPE_LVM_VDO):
            self.skipTest("VDO not supported, skipping")

        self.blivet.factory_device(devicefactory.DEVICE_TYPE_LVM_VDO,
                                   size=Size("10 GiB"),
                                   fstype="ext4",
                                   disks=self.blivet.disks[:],
                                   name="vdolv",
                                   pool_name="vdopool",
                                   virtual_size=Size("40 GiB"))


class StratisTestCase(BlivetResetTestCase):

    def _set_up_storage(self):
        if not devicefactory.is_supported_device_type(devicefactory.DEVICE_TYPE_STRATIS):
            self.skipTest("Stratis not supported, skipping")

        self.blivet.factory_device(devicefactory.DEVICE_TYPE_STRATIS,
                                   size=Size("10 GiB"),
                                   disks=self.blivet.disks[:],
                                   name="stratisfs",
                                   pool_name="stratispool")


@unittest.skip("temporarily disabled due to issues with raids with metadata version 0.90")
class MDRaid0TestCase(BlivetResetTestCase):

    """ Verify correct detection of MD RAID0 arrays. """
    level = "raid0"
    _validate_attrs = BlivetResetTestCase._validate_attrs + ["level", "spares"]

    def skip_attr(self, device, attr):
        return (attr in ["name", "path"] and
                getattr(device, "metadata_version", "").startswith("0.9"))

    def _set_up_storage(self):
        device = self.blivet.factory_device(devicefactory.DEVICE_TYPE_MD,
                                            size=Size("200 MiB"),
                                            disks=self.blivet.disks[:],
                                            raid_level=self.level,
                                            label="v090",
                                            name="2")
        device.metadata_version = "0.90"

        device = self.blivet.factory_device(devicefactory.DEVICE_TYPE_MD,
                                            size=Size("200 MiB"),
                                            disks=self.blivet.disks[:],
                                            raid_level=self.level,
                                            label="v1",
                                            name="one")
        device.metadata_version = "1.0"

        device = self.blivet.factory_device(devicefactory.DEVICE_TYPE_MD,
                                            size=Size("200 MiB"),
                                            disks=self.blivet.disks[:],
                                            raid_level=self.level,
                                            label="v11",
                                            name="oneone")
        device.metadata_version = "1.1"

        device = self.blivet.factory_device(devicefactory.DEVICE_TYPE_MD,
                                            size=Size("200 MiB"),
                                            disks=self.blivet.disks[:],
                                            raid_level=self.level,
                                            label="v12",
                                            name="onetwo")
        device.metadata_version = "1.2"

        device = self.blivet.factory_device(devicefactory.DEVICE_TYPE_MD,
                                            size=Size("200 MiB"),
                                            disks=self.blivet.disks[:],
                                            raid_level=self.level,
                                            label="vdefault",
                                            name="default")


class LVMOnMDTestCase(BlivetResetTestCase):
    # This also tests raid1 with the default metadata version.

    def _set_up_storage(self):
        self.blivet.factory_device(devicefactory.DEVICE_TYPE_LVM,
                                   size=Size("200 MiB"),
                                   disks=self.blivet.disks[:],
                                   container_raid_level="raid1",
                                   fstype="swap")
        self.blivet.factory_device(devicefactory.DEVICE_TYPE_LVM,
                                   disks=self.blivet.disks[:],
                                   container_raid_level="raid1")



# devices/optical.py
#
# Copyright (C) 2009-2014  Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# the GNU General Public License v.2, or (at your option) any later version.
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY expressed or implied, including the implied warranties of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.  You should have received a copy of the
# GNU General Public License along with this program; if not, write to the
# Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.  Any Red Hat trademarks that are incorporated in the
# source code or documentation are not subject to the GNU General Public
# License and may only be used or replicated with the express permission of
# Red Hat, Inc.
#
# Red Hat Author(s): David Lehman <dlehman@redhat.com>
#

import os

from .. import errors
from .. import util
from ..storage_log import log_method_call

import logging
log = logging.getLogger("blivet")

from .storage import StorageDevice


class OpticalDevice(StorageDevice):

    """ An optical drive, eg: cdrom, dvd+r, &c.

        XXX Is this useful?
    """
    _type = "cdrom"

    def __init__(self, name, major=None, minor=None, exists=False,
                 fmt=None, parents=None, sysfs_path='', vendor="",
                 model=""):
        StorageDevice.__init__(self, name, fmt=fmt,
                               major=major, minor=minor, exists=True,
                               parents=parents, sysfs_path=sysfs_path,
                               vendor=vendor, model=model)

    @property
    def media_present(self):
        """ Return a boolean indicating whether or not the device contains
            media.
        """
        log_method_call(self, self.name, status=self.status)
        if not self.exists:
            raise errors.DeviceError("device has not been created")

        try:
            fd = os.open(self.path, os.O_RDONLY)
        except OSError as e:
            # errno 123 = No medium found
            if e.errno == 123:
                return False
            else:
                return True
        else:
            os.close(fd)
            return True

    def eject(self):
        """ Eject the drawer. """
        log_method_call(self, self.name, status=self.status)
        if not self.exists:
            raise errors.DeviceError("device has not been created")

        # try to umount and close device before ejecting
        self.teardown()

        try:
            util.run_program(["eject", self.name])
        except OSError as e:
            log.warning("error ejecting cdrom %s: %s", self.name, e)



import os


def pytest_ignore_collect(path, config):  # pylint: disable=unused-argument
    # we want to ignore symlinks in tests/devicelibs_test/edd_data/
    #  which cause pytest to end in an infinite loop
    if os.path.islink(path):
        return True



# populator.py
# Backend code for populating a DeviceTree.
#
# Copyright (C) 2009-2015  Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# the GNU Lesser General Public License v.2, or (at your option) any later
# version. This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY expressed or implied, including the implied
# warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
# the GNU Lesser General Public License for more details.  You should have
# received a copy of the GNU Lesser General Public License along with this
# program; if not, write to the Free Software Foundation, Inc., 51 Franklin
# Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat trademarks
# that are incorporated in the source code or documentation are not subject
# to the GNU Lesser General Public License and may only be used or
# replicated with the express permission of Red Hat, Inc.
#
# Red Hat Author(s): David Lehman <dlehman@redhat.com>
#

import os
import pprint
import copy
import parted
from six import add_metaclass

import gi
gi.require_version("BlockDev", "3.0")

from gi.repository import BlockDev as blockdev

from ..errors import DeviceError, DeviceTreeError, NoParentsError
from ..devices import DMLinearDevice
from ..devices import FileDevice, LoopDevice
from ..devices import MDRaidArrayDevice
from ..devices import MultipathDevice
from ..devices import NoDevice
from ..devicelibs import disk as disklib
from ..devicelibs import lvm
from .. import formats
from .. import udev
from .. import util
from ..flags import flags
from ..storage_log import log_method_call
from ..tasks import availability
from ..threads import SynchronizedMeta
from .helpers import get_device_helper, get_format_helper
from ..static_data import lvs_info, pvs_info, vgs_info, luks_data, mpath_members, stratis_info
from ..callbacks import callbacks

import logging
log = logging.getLogger("blivet")


def parted_exn_handler(exn_type, exn_options, exn_msg):
    """ Answer any of parted's yes/no questions in the affirmative.

        This allows us to proceed with partially corrupt gpt disklabels.
    """
    log.info("parted exception: %s", exn_msg)
    ret = parted.EXCEPTION_RESOLVE_UNHANDLED
    if exn_type == parted.EXCEPTION_TYPE_ERROR and \
       exn_options == parted.EXCEPTION_OPT_YES_NO:
        ret = parted.EXCEPTION_RESOLVE_YES
    elif exn_type == parted.EXCEPTION_TYPE_WARNING and \
            exn_options & parted.EXCEPTION_RESOLVE_FIX:
        ret = parted.EXCEPTION_RESOLVE_FIX
    return ret


@add_metaclass(SynchronizedMeta)
class PopulatorMixin(object):
    def __init__(self, disk_images=None):
        """
            :keyword disk_images: dictoinary of disk images
            :type list: dict
        """
        luks_data.reset(passphrase=None, luks_dict={})
        self.reset(disk_images=disk_images)

    def reset(self, disk_images=None):
        luks_data.reset()
        self.disk_images = {}
        if disk_images:
            # this will overwrite self.exclusive_disks
            self.set_disk_images(disk_images)

        # initialize attributes that may later hold cached lvm info
        self.drop_device_info_cache()

        self._cleanup = False

    def _add_parent_devices(self, info):
        """ Add all parents of a device, raising DeviceTreeError on failure.

            :param :class:`pyudev.Device` info: the device's udev info
            :raises: :class:`~.errors.DeviceTreeError if no parents are found or
                     if we fail to add any parent
            :returns: a list of parent devices
            :rtype: list of :class:`~.StorageDevice`
        """
        name = udev.device_get_name(info)
        sysfs_path = udev.device_get_sysfs_path(info)
        parent_dir = os.path.normpath("%s/slaves" % sysfs_path)
        parent_names = os.listdir(parent_dir)
        parent_devices = []
        if not parent_names:
            log.error("no parents found for %s", name)
            raise NoParentsError("no parents found for device %s" % name)

        for parent_name in parent_names:
            path = os.path.normpath("%s/%s" % (parent_dir, parent_name))
            parent_info = udev.get_device(os.path.realpath(path))

            if not parent_info:
                msg = "unable to get udev info for %s" % parent_name
                raise DeviceTreeError(msg)

            # cciss in sysfs is "cciss!cXdYpZ" but we need "cciss/cXdYpZ"
            parent_name = udev.device_get_name(parent_info).replace("!", "/")

            parent_dev = self.get_device_by_name(parent_name)
            if not parent_dev and parent_info:
                # we haven't scanned the parent yet, so do it now
                self.handle_device(parent_info)
                parent_dev = self.get_device_by_name(parent_name)
                if parent_dev is None:
                    if udev.device_is_dm_lvm(info):
                        if parent_name not in lvs_info.cache:
                            # we do not expect hidden lvs to be in the tree
                            continue

                    # if the current parent is still not in
                    # the tree, something has gone wrong
                    log.error("failure scanning device %s: could not add parent %s", name, parent_name)
                    msg = "failed to add parent %s of device %s" % (parent_name, name)
                    raise DeviceTreeError(msg)

            parent_devices.append(parent_dev)

        return parent_devices

    def _add_name(self, name):
        if name not in self.names:
            self.names.append(name)

    def _remove_name(self, name):
        if name in self.names:
            self.names.remove(name)

    def _reason_to_skip_device(self, info):
        sysfs_path = udev.device_get_sysfs_path(info)
        uuid = udev.device_get_uuid(info)

        # make sure this device was not scheduled for removal and also has not
        # been hidden
        removed = [a.device for a in self.actions.find(action_type="destroy",
                                                       object_type="device")]
        for ignored in removed + self._hidden:
            if (sysfs_path and ignored.sysfs_path == sysfs_path) or \
               (uuid and uuid in (ignored.uuid, ignored.format.uuid)):
                if ignored in removed:
                    reason = "removed"
                else:
                    reason = "hidden"

                return reason

    def _handle_degraded_md(self, info, device):
        if device is not None or not udev.device_is_md(info):
            return device

        # If the md name is None, then some udev info is missing. Likely,
        # this is because the array is degraded, and mdadm has deactivated
        # it. Try to activate it and re-get the udev info.
        if flags.allow_imperfect_devices and udev.device_get_md_name(info) is None:
            devname = udev.device_get_devname(info)
            if devname:
                try:
                    blockdev.md.run(devname)
                except blockdev.MDRaidError as e:
                    log.warning("Failed to start possibly degraded md array: %s", e)
                else:
                    udev.settle()
                    info = udev.get_device(udev.device_get_sysfs_path(info))
            else:
                log.warning("Failed to get devname for possibly degraded md array.")

        md_name = udev.device_get_md_name(info)
        if md_name is None:
            log.warning("No name for possibly degraded md array.")
        else:
            device = self.get_device_by_name(md_name, incomplete=flags.allow_imperfect_devices)

        if device and not isinstance(device, MDRaidArrayDevice):
            log.warning("Found device %s, but it turns out not be an md array device after all.", device.name)
            device = None

        return device

    def _clear_new_multipath_member(self, device):
        if device is None or not device.is_disk or not mpath_members.is_mpath_member(device.path):
            return

        # newly added device (eg iSCSI) could make this one a multipath member
        if device.format.type != "multipath_member":
            log.debug("%s newly detected as multipath member, dropping old format and removing kids", device.name)
            # remove children from tree so that we don't stumble upon them later
            self.recursive_remove(device, actions=False, remove_device=False)

    def _mark_readonly_device(self, info, device):
        # If this device is read-only, mark it as such now.
        if udev.device_is_disk(info) and \
                util.get_sysfs_attr(udev.device_get_sysfs_path(info), 'ro') == '1':
            device.readonly = True

    def _update_exclusive_disks(self, device):
        # If we just added a multipath or fwraid disk that is in exclusive_disks
        # we have to make sure all of its members are in the list too.
        mdclasses = (MDRaidArrayDevice, MultipathDevice)
        if device.is_disk and isinstance(device, mdclasses):
            if device.name in self.exclusive_disks:
                for ancestor in device.ancestors:
                    if ancestor.is_disk and ancestor.name not in self.exclusive_disks:
                        self.exclusive_disks.append(ancestor.name)

    def _get_format_helper(self, info, device=None):
        return get_format_helper(info, device=device)

    def _get_device_helper(self, info):
        return get_device_helper(info)

    def handle_device(self, info, update_orig_fmt=False):
        """
            :param :class:`pyudev.Device` info: udev info for the device
            :keyword bool update_orig_fmt: update original format unconditionally

            If a device is added to the tree based on info its original format
            will be saved after the format has been detected. If the device
            that corresponds to info is already in the tree, its original format
            will not be updated unless update_orig_fmt is True.
        """
        name = udev.device_get_name(info)
        log_method_call(self, name=name, info=pprint.pformat(dict(info)))
        sysfs_path = udev.device_get_sysfs_path(info)

        reason = self._reason_to_skip_device(info)
        if reason:
            log.debug("skipping %s device %s", reason, name)
            callbacks.device_scanned(device_name=name)
            return

        log.info("scanning %s (%s)...", name, sysfs_path)
        if udev.device_is_hidden(info):
            log.info("device %s is marked as hidden in sysfs, ignoring", name)
            return

        if udev.device_is_private(info):
            log.info("device %s is private, ignoring", name)
            return

        if udev.device_is_stratis_filesystem(info):
            log.debug("skipping %s stratis filesystem, will be handled with corresponding "
                      "stratis block device", name)
            return

        if udev.device_is_dm_luks(info) and udev.device_is_stratis_private(info):
            log.debug("skipping %s stratis private crypto device, will be handled with "
                      "corresponding stratis block device", name)

            # XXX for newly unlocked stratis pools we need to force run handle_format
            # on one of the parent block devices to add the pool to the tree
            parents = udev.device_get_parents(info)
            if parents:
                bname = udev.device_get_name(parents[0])
                bdev = self.get_device_by_name(bname)
                if bdev:
                    self.handle_format(info, bdev, force=True)
            return

        # make sure we note the name of every device we see
        self._add_name(name)
        device = self.get_device_by_name(name)
        device = self._handle_degraded_md(info, device)
        self._clear_new_multipath_member(device)

        device_added = True
        helper_class = None
        if device:
            device_added = False
        else:
            helper_class = self._get_device_helper(info)

        if helper_class is not None:
            device = helper_class(self, info).run()

        if not device:
            log.debug("no device obtained for %s", name)
            callbacks.device_scanned(device_name=name)
            return

        log.info("got device: %r", device)
        self._mark_readonly_device(info, device)
        self._update_exclusive_disks(device)

        # now handle the device's formatting
        self.handle_format(info, device)
        if device_added or update_orig_fmt:
            device.original_format = copy.deepcopy(device.format)
        device.device_links = udev.device_get_symlinks(info)
        callbacks.device_scanned(device_name=name)

    def handle_format(self, info, device, force=False):
        log_method_call(self, name=getattr(device, "name", None))

        if not info:
            info = udev.get_device(device.sysfs_path)
            if not info:
                log.debug("no information for device %s", device.name)
                return
        if not device.media_present:
            log.debug("no media present for device %s", device.name)
            return

        name = udev.device_get_name(info)
        if (not device or
            (not udev.device_get_format(info) and not udev.device_get_disklabel_type(info)) or
           (device.format.type and not force)):
            # this device has no formatting or it has already been set up
            log.debug("no type or existing type for %s, bailing", name)
            return

        helper_class = self._get_format_helper(info, device=device)
        if helper_class is not None:
            helper_class(self, info, device).run()

        log.info("got format: %s", device.format)

    def _handle_inconsistencies(self):
        for vg in [d for d in self.devices if d.type == "lvmvg"]:
            if vg.complete:
                continue

            # Make sure lvm doesn't get confused by PVs that belong to
            # incomplete VGs. We will add the PVs to the accept list when/if
            # the time comes to remove the incomplete VG and its PVs.
            for pv in vg.pvs:
                lvm.lvm_devices_remove(pv.path)

    def set_disk_images(self, images):
        """ Set the disk images and reflect them in exclusive_disks.

            :param images: dict with image name keys and filename values
            :type images: dict

            .. note::

                Disk images are automatically exclusive. That means that, in the
                presence of disk images, any local storage not associated with
                the disk images is ignored.
        """
        self.disk_images = images
        # disk image files are automatically exclusive
        self.exclusive_disks = list(self.disk_images.keys())

    def setup_disk_images(self):
        """ Set up devices to represent the disk image files. """
        for (name, path) in self.disk_images.items():
            log.info("setting up disk image file '%s' as '%s'", path, name)
            dmdev = self.get_device_by_name(name)
            if dmdev and isinstance(dmdev, DMLinearDevice) and \
               path in (d.path for d in dmdev.ancestors):
                log.debug("using %s", dmdev)
                dmdev.setup()
                continue

            try:
                filedev = FileDevice(path, exists=True)
                filedev.setup()
                log.debug("%s", filedev)

                loop_name = blockdev.loop.get_loop_name(filedev.path)
                loop_sysfs = None
                if loop_name:
                    loop_sysfs = "/class/block/%s" % loop_name
                loopdev = LoopDevice(name=loop_name,
                                     parents=[filedev],
                                     sysfs_path=loop_sysfs,
                                     exists=True)
                loopdev.setup()
                log.debug("%s", loopdev)
                dmdev = DMLinearDevice(name,
                                       dm_uuid="ANACONDA-%s" % name,
                                       parents=[loopdev],
                                       exists=True)
                dmdev.setup()
                dmdev.update_sysfs_path()
                dmdev.update_size()
                log.debug("%s", dmdev)
            except (ValueError, DeviceError) as e:
                log.error("failed to set up disk image: %s", e)
            else:
                self._add_device(filedev)
                self._add_device(loopdev)
                self._add_device(dmdev)
                info = udev.get_device(dmdev.sysfs_path)
                self.handle_device(info, update_orig_fmt=True)

    def teardown_disk_images(self):
        """ Tear down any disk image stacks. """
        if not self.disk_images:
            return

        self.teardown_all()
        for (name, _path) in self.disk_images.items():
            dm_device = self.get_device_by_name(name)
            if not dm_device:
                continue

            dm_device.deactivate()
            loop_device = dm_device.parents[0]
            loop_device.teardown()

    def save_luks_passphrase(self, device):
        """ Save a device's LUKS passphrase in case of reset. """
        # Method is here for compatibility with blivet 1.x
        luks_data.save_passphrase(device)

    def populate(self, cleanup_only=False):
        """ Locate all storage devices.

            Everything should already be active. We just go through and gather
            details as needed and set up the relations between various devices.

            Devices excluded via disk filtering (or because of disk images) are
            scanned just the rest, but then they are hidden at the end of this
            process.
        """
        if cleanup_only:
            self._cleanup = True

        parted.register_exn_handler(parted_exn_handler)
        try:
            self._populate()
        finally:
            parted.clear_exn_handler()
            self._hide_ignored_disks()

    def _populate(self):
        log.info("DeviceTree.populate: ignored_disks is %s ; exclusive_disks is %s",
                 self.ignored_disks, self.exclusive_disks)

        disklib.update_volume_info()
        self.drop_device_info_cache()

        if flags.auto_dev_updates and availability.BLOCKDEV_MPATH_PLUGIN.available:
            blockdev.mpath.set_friendly_names(flags.multipath_friendly_names)

        self.setup_disk_images()

        old_devices = {}
        n_devices = 0
        report = True

        # Now, loop and scan for devices that have appeared since the two above
        # blocks or since previous iterations.
        while True:
            devices = []
            new_devices = udev.get_devices()

            for new_device in new_devices:
                new_name = udev.device_get_name(new_device)
                if new_name not in old_devices:
                    old_devices[new_name] = new_device
                    n_devices += 1
                    devices.append(new_device)

            if len(devices) == 0:
                # nothing is changing -- we are finished building devices
                break

            if report:
                callbacks.populate_started(n_devices=n_devices)
                report = False

            log.info("devices to scan: %s", [udev.device_get_name(d) for d in devices])
            for dev in devices:
                self.handle_device(dev)

        # After having the complete tree we make sure that the system
        # inconsistencies are ignored or resolved.
        self._handle_inconsistencies()

    def drop_device_info_cache(self):
        """ Drop cached device information. """
        lvs_info.drop_cache()
        pvs_info.drop_cache()
        vgs_info.drop_cache()
        mpath_members.drop_cache()
        stratis_info.drop_cache()

    def handle_nodev_filesystems(self):
        for line in open("/proc/mounts").readlines():
            try:
                (_devspec, mountpoint, fstype, _options, _rest) = line.split(None, 4)
            except ValueError:
                log.error("failed to parse /proc/mounts line: %s", line)
                continue
            if fstype in formats.fslib.nodev_filesystems:
                if not flags.include_nodev:
                    continue

                log.info("found nodev %s filesystem mounted at %s",
                         fstype, mountpoint)
                # nodev filesystems require some special handling.
                # For now, a lot of this is based on the idea that it's a losing
                # battle to require the presence of an FS class for every type
                # of nodev filesystem. Based on that idea, we just instantiate
                # NoDevFS directly and then hack in the fstype as the device
                # attribute.
                fmt = formats.get_format("nodev")
                fmt.device = fstype

                # NoDevice also needs some special works since they don't have
                # per-instance names in the kernel.
                device = NoDevice(fmt=fmt)
                n = len([d for d in self.devices if d.format.type == fstype])
                device._name += ".%d" % n
                self._add_device(device)



# populator/helpers/dm.py
# Device-mapper backend code for populating a DeviceTree.
#
# Copyright (C) 2009-2015  Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# the GNU Lesser General Public License v.2, or (at your option) any later
# version. This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY expressed or implied, including the implied
# warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
# the GNU Lesser General Public License for more details.  You should have
# received a copy of the GNU Lesser General Public License along with this
# program; if not, write to the Free Software Foundation, Inc., 51 Franklin
# Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat trademarks
# that are incorporated in the source code or documentation are not subject
# to the GNU Lesser General Public License and may only be used or
# replicated with the express permission of Red Hat, Inc.
#
# Red Hat Author(s): David Lehman <dlehman@redhat.com>
#

from ... import udev
from ...devices import DMDevice
from ...storage_log import log_method_call
from .devicepopulator import DevicePopulator

import logging
log = logging.getLogger("blivet")


class DMDevicePopulator(DevicePopulator):
    priority = 50

    @classmethod
    def match(cls, data):
        return (udev.device_is_dm(data) and
                not udev.device_is_dm_partition(data) and
                not udev.device_is_dm_luks(data) and
                not udev.device_is_dm_integrity(data) and
                not udev.device_is_dm_lvm(data) and
                not udev.device_is_dm_mpath(data) and
                not udev.device_is_dm_raid(data) and
                not udev.device_is_dm_stratis(data))

    def run(self):
        name = udev.device_get_name(self.data)
        log_method_call(self, name=name)
        sysfs_path = udev.device_get_sysfs_path(self.data)
        parent_devices = self._devicetree._add_parent_devices(self.data)
        device = self._devicetree.get_device_by_name(name)

        if device is None:
            device = DMDevice(name, dm_uuid=self.data.get('DM_UUID'),
                              sysfs_path=sysfs_path, exists=True,
                              parents=[parent_devices[0]])
            device.protected = True
            device.controllable = False
            self._devicetree._add_device(device)

        return device



try:
    from unittest.mock import patch, PropertyMock
except ImportError:
    from mock import patch, PropertyMock

import unittest
import os
import six
import blivet
import gi
gi.require_version("BlockDev", "3.0")
from gi.repository import BlockDev as blockdev

from blivet.errors import DependencyError

from blivet.deviceaction import ActionCreateDevice
from blivet.deviceaction import ActionDestroyDevice
from blivet.deviceaction import ActionResizeDevice

from blivet.deviceaction import ActionCreateFormat
from blivet.deviceaction import ActionDestroyFormat

from blivet.devices import DiskDevice
from blivet.devices import DiskFile
from blivet.devices import LUKSDevice
from blivet.devices import MDRaidArrayDevice
from blivet.devices import PartitionDevice
from blivet.devices import StorageDevice

from blivet.formats import get_format

from blivet.size import Size

from blivet.tasks import availability

from blivet.util import create_sparse_tempfile


class DeviceDependenciesTestCase(unittest.TestCase):

    """Test external device dependencies. """

    def test_dependencies(self):
        dev1 = DiskDevice("name", fmt=get_format("mdmember"))
        dev2 = DiskDevice("other", fmt=get_format("mdmember"))
        dev = MDRaidArrayDevice("dev", level="raid1", parents=[dev1, dev2])
        luks = LUKSDevice("luks", parents=[dev])

        # a parent's dependencies are a subset of its child's.
        for d in dev.external_dependencies:
            self.assertIn(d, luks.external_dependencies)

        # make sure that there's at least something in these dependencies
        self.assertGreater(len(luks.external_dependencies), 0)


class MockingDeviceDependenciesTestCase1(unittest.TestCase):

    """Test availability of external device dependencies. """

    def setUp(self):
        dev1 = DiskDevice("name", fmt=get_format("mdmember"), size=Size("1 GiB"))
        dev2 = DiskDevice("other")
        self.part = PartitionDevice("part", fmt=get_format("mdmember"), parents=[dev2])
        self.dev = MDRaidArrayDevice("dev", level="raid1", parents=[dev1, self.part], fmt=get_format("luks"), total_devices=2, member_devices=2)
        self.luks = LUKSDevice("luks", parents=[self.dev], fmt=get_format("ext4"))

        self.mdraid_method = availability.BLOCKDEV_MDRAID_PLUGIN._method
        self.dm_method = availability.BLOCKDEV_DM_PLUGIN._method
        self.hfsplus_method = availability.MKFS_HFSPLUS_APP._method
        self.cache_availability = availability.CACHE_AVAILABILITY

        self.addCleanup(self._clean_up)

    def test_availability_mdraidplugin(self):

        availability.CACHE_AVAILABILITY = False
        availability.BLOCKDEV_DM_PLUGIN._method = availability.AvailableMethod

        # if the plugin is not in, there's nothing to test
        self.assertIn(availability.BLOCKDEV_MDRAID_PLUGIN, self.luks.external_dependencies)

        # dev is not among its unavailable dependencies
        availability.BLOCKDEV_MDRAID_PLUGIN._method = availability.AvailableMethod
        availability.MKFS_HFSPLUS_APP._method = availability.AvailableMethod  # macefi
        availability.BLOCKDEV_CRYPTO_PLUGIN._method = availability.AvailableMethod  # luks
        availability.KPARTX_APP._method = availability.AvailableMethod  # luks
        self.assertNotIn(availability.BLOCKDEV_MDRAID_PLUGIN, self.luks.unavailable_dependencies)
        self.assertIsNotNone(ActionCreateDevice(self.luks))
        self.assertIsNotNone(ActionDestroyDevice(self.luks))
        self.assertIsNotNone(ActionCreateFormat(self.luks, fmt=get_format("macefi")))
        self.assertIsNotNone(ActionDestroyFormat(self.luks))

        # dev is among the unavailable dependencies
        availability.BLOCKDEV_MDRAID_PLUGIN._method = availability.UnavailableMethod
        self.assertIn(availability.BLOCKDEV_MDRAID_PLUGIN, self.luks.unavailable_dependencies)
        with self.assertRaises(DependencyError):
            ActionDestroyDevice(self.dev)

    def _clean_up(self):
        availability.BLOCKDEV_MDRAID_PLUGIN._method = self.mdraid_method
        availability.BLOCKDEV_DM_PLUGIN._method = self.dm_method
        availability.MKFS_HFSPLUS_APP._method = self.hfsplus_method

        availability.CACHE_AVAILABILITY = False
        availability.BLOCKDEV_MDRAID_PLUGIN.available  # pylint: disable=pointless-statement
        availability.BLOCKDEV_DM_PLUGIN.available  # pylint: disable=pointless-statement
        availability.MKFS_HFSPLUS_APP.available  # pylint: disable=pointless-statement

        availability.CACHE_AVAILABILITY = self.cache_availability


class MockingDeviceDependenciesTestCase2(unittest.TestCase):
    def test_dependencies_handling(self):
        device = StorageDevice("testdev1")
        self.assertTrue(device.controllable)
        self.assertIsNotNone(ActionCreateDevice(device))
        device.exists = True
        self.assertIsNotNone(ActionDestroyDevice(device))
        with patch.object(StorageDevice, "resizable", new_callable=PropertyMock(return_value=True)):
            self.assertIsNotNone(ActionResizeDevice(device, Size("1 GiB")))

        # if any external dependency is missing, it should be impossible to create, destroy, setup,
        # teardown, or resize the device (controllable encompasses setup & teardown)
        with patch.object(StorageDevice, "_external_dependencies",
                          new_callable=PropertyMock(return_value=[availability.unavailable_resource("testing")])):
            device = StorageDevice("testdev1")
            self.assertFalse(device.controllable)
            self.assertRaises(DependencyError, ActionCreateDevice, device)
            device.exists = True
            self.assertRaises(DependencyError, ActionDestroyDevice, device)
            self.assertRaises(ValueError, ActionResizeDevice, device, Size("1 GiB"))

        # same goes for formats, except that the properties they affect vary by format class
        fmt = get_format("lvmpv")
        fmt._plugin = availability.available_resource("lvm-testing")
        self.assertTrue(fmt.supported)
        self.assertTrue(fmt.formattable)
        self.assertTrue(fmt.destroyable)

        fmt._plugin = availability.unavailable_resource("lvm-testing")
        self.assertFalse(fmt.supported)
        self.assertFalse(fmt.formattable)
        self.assertFalse(fmt.destroyable)


class MissingWeakDependenciesTestCase(unittest.TestCase):

    def setUp(self):
        self.addCleanup(self._clean_up)
        self.disk1_file = create_sparse_tempfile("disk1", Size("2GiB"))
        self.plugins = blockdev.plugin_specs_from_names(blockdev.get_available_plugin_names())  # pylint: disable=no-value-for-parameter

        loaded_plugins = self.load_all_plugins()
        if not all(p in loaded_plugins for p in ("btrfs", "crypto", "lvm", "md")):
            # we don't have all plugins needed for this test case
            self.skipTest("Missing libblockdev plugins needed from weak dependencies test.")

    def _clean_up(self):
        # reload all libblockdev plugins
        self.load_all_plugins()

        if os.path.exists(self.disk1_file):
            os.unlink(self.disk1_file)

        availability.CACHE_AVAILABILITY = True

    def load_all_plugins(self):
        result, plugins = blockdev.try_reinit(require_plugins=self.plugins, reload=True)
        if not result:
            self.fail("Could not reload libblockdev plugins")
        return plugins

    def unload_all_plugins(self):
        result, _ = blockdev.try_reinit(require_plugins=[], reload=True)
        if not result:
            self.fail("Could not reload libblockdev plugins")

    def test_weak_dependencies(self):
        self.bvt = blivet.Blivet()  # pylint: disable=attribute-defined-outside-init
        availability.CACHE_AVAILABILITY = False

        # reinitialize blockdev without the plugins
        # TODO: uncomment (workaround (1/2) for blivet.reset fail)
        # self.unload_all_plugins()
        disk1 = DiskFile(self.disk1_file)

        self.bvt.exclusive_disks = [disk1.name]
        if os.geteuid() == 0:
            try:
                self.bvt.reset()
            except blockdev.BlockDevNotImplementedError:  # pylint: disable=catching-non-exception
                self.fail("Improper handling of missing libblockdev plugin")
        # TODO: remove line (workaround (2/2) for blivet.reset fail)
        self.unload_all_plugins()

        self.bvt.devicetree._add_device(disk1)
        self.bvt.initialize_disk(disk1)

        pv = self.bvt.new_partition(size=Size("8GiB"), fmt_type="lvmpv")
        pv_fail = self.bvt.new_partition(size=Size("8GiB"), fmt_type="lvmpv")
        btrfs = self.bvt.new_partition(size=Size("1GiB"), fmt_type="btrfs")
        raid1 = self.bvt.new_partition(size=Size("1GiB"), fmt_type="mdmember")
        raid2 = self.bvt.new_partition(size=Size("1GiB"), fmt_type="mdmember")

        with six.assertRaisesRegex(self, ValueError, "resource to create this format.*unavailable"):
            self.bvt.create_device(pv_fail)

        # to be able to test functions like destroy_device it is necessary to have some
        # testing devices actually created - hence loading libblockdev plugins...
        self.load_all_plugins()
        self.bvt.create_device(pv)
        self.bvt.create_device(btrfs)
        # ... and unloading again when tests can continue
        self.unload_all_plugins()

        try:
            vg = self.bvt.new_vg(parents=[pv])
        except blockdev.BlockDevNotImplementedError:  # pylint: disable=catching-non-exception
            self.fail("Improper handling of missing libblockdev plugin")

        self.load_all_plugins()
        self.bvt.create_device(vg)
        self.unload_all_plugins()

        try:
            lv1 = self.bvt.new_lv(fmt_type="ext4", size=Size("1GiB"), parents=[vg])
            lv2 = self.bvt.new_lv(fmt_type="ext4", size=Size("1GiB"), parents=[vg])
            lv3 = self.bvt.new_lv(fmt_type="luks", size=Size("1GiB"), parents=[vg])
        except blockdev.BlockDevNotImplementedError:  # pylint: disable=catching-non-exception
            self.fail("Improper handling of missing libblockdev plugin")

        self.load_all_plugins()
        self.bvt.create_device(lv1)
        self.bvt.create_device(lv2)
        self.bvt.create_device(lv3)
        self.unload_all_plugins()

        try:
            pool = self.bvt.new_lv_from_lvs(vg, name='pool', seg_type="thin-pool", from_lvs=[lv1, lv2])
        except blockdev.BlockDevNotImplementedError:  # pylint: disable=catching-non-exception
            self.fail("Improper handling of missing libblockdev plugin")

        self.load_all_plugins()
        self.bvt.create_device(pool)
        self.unload_all_plugins()

        with six.assertRaisesRegex(self, DependencyError, "requires unavailable_dependencies"):
            self.bvt.destroy_device(pool)

        try:
            self.bvt.new_btrfs(parents=[btrfs])
        except blockdev.BlockDevNotImplementedError:  # pylint: disable=catching-non-exception
            self.fail("Improper handling of missing libblockdev plugin")

        with six.assertRaisesRegex(self, ValueError, "device cannot be resized"):
            self.bvt.resize_device(lv3, Size("2GiB"))

        try:
            self.bvt.new_tmp_fs(fmt=disk1.format, size=Size("500MiB"))
        except blockdev.BlockDevNotImplementedError:  # pylint: disable=catching-non-exception
            self.fail("Improper handling of missing libblockdev plugin")

        try:
            self.bvt.new_mdarray(level='raid0', parents=[raid1, raid2])
        except blockdev.BlockDevNotImplementedError:  # pylint: disable=catching-non-exception
            self.fail("Improper handling of missing libblockdev plugin")



import six
import unittest

try:
    from unittest.mock import patch
except ImportError:
    from mock import patch

import blivet

from blivet.devices import StorageDevice
from blivet.devices import StratisPoolDevice
from blivet.devices import StratisFilesystemDevice
from blivet.errors import StratisError
from blivet.size import Size


DEVICE_CLASSES = [
    StratisPoolDevice,
    StratisFilesystemDevice,
    StorageDevice
]


@unittest.skipUnless(not any(x.unavailable_type_dependencies() for x in DEVICE_CLASSES), "some unsupported device classes required for this test")
class BlivetNewStratisDeviceTest(unittest.TestCase):
    def test_new_stratis(self):
        b = blivet.Blivet()
        bd = StorageDevice("bd1", fmt=blivet.formats.get_format("stratis"),
                           size=Size("2 GiB"), exists=False)

        b.devicetree._add_device(bd)

        with patch("blivet.devicetree.DeviceTree.names", []):
            pool = b.new_stratis_pool(name="testpool", parents=[bd])
        self.assertEqual(pool.name, "testpool")
        self.assertEqual(pool.size, bd.size)

        with patch("blivet.devicelibs.stratis.pool_used", lambda _d, _e: Size("512 MiB")):
            self.assertAlmostEqual(pool.free_space, Size("1.5 GiB"))

        with patch("blivet.devicetree.DeviceTree.names", []):
            fs = b.new_stratis_filesystem(name="testfs", parents=[pool], size=Size("1 GiB"))

        self.assertEqual(fs.name, "testpool/testfs")
        self.assertEqual(fs.path, "/dev/stratis/%s" % fs.name)
        self.assertEqual(fs.size, Size("1 GiB"))
        self.assertEqual(fs.pool, pool)
        self.assertEqual(fs.format.type, "stratis xfs")
        # for 1 TiB filesystem, metadata should take around 1 GiB
        self.assertAlmostEqual(fs.used_size, Size("20 MiB"), delta=Size("1 MiB"))

        with patch("blivet.devicetree.DeviceTree.names", []):
            with six.assertRaisesRegex(self, StratisError, "not enough free space in the pool"):
                # not enough free space for a 2 TiB filesystem
                b.new_stratis_filesystem(name="testfs2", parents=[pool], size=Size("2 TiB"))

        b.create_device(pool)
        b.create_device(fs)

        with patch("blivet.devicelibs.stratis") as stratis_dbus:
            with patch.object(pool, "_pre_create"):
                with patch.object(pool, "_post_create"):
                    pool.create()
                    stratis_dbus.create_pool.assert_called_with(name='testpool',
                                                                devices=['/dev/bd1'],
                                                                encrypted=False,
                                                                passphrase=None,
                                                                key_file=None)

        # we would get this from pool._post_create
        pool.uuid = "c4fc9ebe-e173-4cab-8d81-cc6abddbe02d"

        with patch("blivet.devicelibs.stratis") as stratis_dbus:
            with patch.object(fs, "_pre_create"):
                with patch.object(fs, "_post_create"):
                    fs.create()
                    stratis_dbus.create_filesystem.assert_called_with(name="testfs",
                                                                      pool_uuid="c4fc9ebe-e173-4cab-8d81-cc6abddbe02d",
                                                                      fs_size=Size("1 GiB"))

    def test_new_encrypted_stratis(self):
        b = blivet.Blivet()
        bd = StorageDevice("bd1", fmt=blivet.formats.get_format("stratis"),
                           size=Size("1 GiB"), exists=True)

        b.devicetree._add_device(bd)

        with patch("blivet.devicetree.DeviceTree.names", []):
            pool = b.new_stratis_pool(name="testpool", parents=[bd], encrypted=True, passphrase="secret")
        self.assertEqual(pool.name, "testpool")
        self.assertEqual(pool.size, bd.size)
        self.assertTrue(pool.encrypted)
        self.assertTrue(pool.has_key)

        b.create_device(pool)

        with patch("blivet.devicelibs.stratis") as stratis_dbus:
            with patch.object(pool, "_pre_create"):
                with patch.object(pool, "_post_create"):
                    pool.create()
                    stratis_dbus.create_pool.assert_called_with(name='testpool',
                                                                devices=['/dev/bd1'],
                                                                encrypted=True,
                                                                passphrase="secret",
                                                                key_file=None)



# Copyright (C) 2015  Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# the GNU Lesser General Public License v.2, or (at your option) any later
# version. This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY expressed or implied, including the implied
# warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
# the GNU Lesser General Public License for more details.  You should have
# received a copy of the GNU Lesser General Public License along with this
# program; if not, write to the Free Software Foundation, Inc., 51 Franklin
# Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat trademarks
# that are incorporated in the source code or documentation are not subject
# to the GNU Lesser General Public License and may only be used or
# replicated with the express permission of Red Hat, Inc.
#
# Red Hat Author(s): David Shea <dshea@redhat.com>

import unittest
import unittest.mock
from polib import POEntry, POFile
import tempfile

from translation_canary.translatable.test_markup import test_markup
from translation_canary.translatable.test_comment import test_comment

from translation_canary.translatable import testString, testPOT

class TestMarkup(unittest.TestCase):
    def test_ok(self):
        # no markup
        test_markup(POEntry(msgid="test string"))

        # internal markup
        test_markup(POEntry(msgid="<b>test</b> string"))

    def test_unnecessary_markup(self):
        self.assertRaises(AssertionError, test_markup, POEntry(msgid="<b>test string</b>"))

class TestComment(unittest.TestCase):
    def test_ok(self):
        # Perfectly fine string
        test_comment(POEntry(msgid="Hello, I am a test string"))

        # single-character string with a comment
        test_comment(POEntry(msgid="c", comment="TRANSLATORS: 'c' to continue"))

    def test_no_comment(self):
        self.assertRaises(AssertionError, test_comment, POEntry(msgid="c"))

# Test the top-level functions
# fake tests for testing with
def _true_test(_s):
    pass

def _false_test(_s):
    raise AssertionError("no good")

def _picky_test(s):
    if s.msgstr.startswith("p"):
        raise AssertionError("I don't like this one")

class TestTestString(unittest.TestCase):
    @unittest.mock.patch("translation_canary.translatable._tests", [_true_test])
    def test_success(self):
        self.assertTrue(testString(POEntry()))

    @unittest.mock.patch("translation_canary.translatable._tests", [_false_test])
    def test_failure(self):
        self.assertFalse(testString(POEntry()))

@unittest.mock.patch("translation_canary.translatable._tests", [_picky_test])
class TestTestPOT(unittest.TestCase):
    def test_success(self):
        with tempfile.NamedTemporaryFile(suffix=".pot") as potfile:
            poobj = POFile()
            poobj.append(POEntry(msgstr="test string"))
            poobj.save(potfile.name)

            self.assertTrue(testPOT(potfile.name))

    def test_some_failure(self):
        with tempfile.NamedTemporaryFile(suffix=".pot") as potfile:
            poobj = POFile()
            poobj.append(POEntry(msgstr="test string"))
            poobj.append(POEntry(msgstr="pest string"))
            poobj.save(potfile.name)

            self.assertFalse(testPOT(potfile.name))

    def test_all_failure(self):
        with tempfile.NamedTemporaryFile(suffix=".pot") as potfile:
            poobj = POFile()
            poobj.append(POEntry(msgstr="pest string"))
            poobj.append(POEntry(msgstr="past string"))
            poobj.save(potfile.name)

            self.assertFalse(testPOT(potfile.name))



import os

import blivet
from blivet.size import Size
from blivet.util import set_up_logging, create_sparse_tempfile
from blivet.devices import PartitionDevice

set_up_logging()
b = blivet.Blivet()   # create an instance of Blivet (don't add system devices)

# create two disk image files on which to create new devices
disk1_file = create_sparse_tempfile("disk1", Size("100GiB"))
b.disk_images["disk1"] = disk1_file

b.reset()

try:
    disk1 = b.devicetree.get_device_by_name("disk1")

    fmt = blivet.formats.get_format("disklabel", device=disk1.path,
                                    label_type="msdos")
    ac = blivet.deviceaction.ActionCreateFormat(device=disk1, fmt=fmt)
    b.devicetree.actions.add(ac)

    # new 5 GiB partition on disk1
    dev = PartitionDevice(name="req%d" % b.next_id,
                          size=Size("5 GiB"), parents=[disk1])

    # low level actions API is usually hidden when using create_device() and format_device()
    # helper functions, working with actions is useful when offering undo/redo functionality
    ac = blivet.deviceaction.ActionCreateDevice(device=dev)
    b.devicetree.actions.add(ac)

    fmt = blivet.formats.get_format("ext4", device=dev.path)
    ac = blivet.deviceaction.ActionCreateFormat(device=dev, fmt=fmt)
    b.devicetree.actions.add(ac)

    # remove last action and format the partition to XFS instead
    b.devicetree.actions.remove(ac)

    fmt = blivet.formats.get_format("xfs", device=dev.path)
    ac = blivet.deviceaction.ActionCreateFormat(device=dev, fmt=fmt)
    b.devicetree.actions.add(ac)

    # now remove the format completely
    ac = blivet.deviceaction.ActionDestroyFormat(device=dev)
    b.devicetree.actions.add(ac)

    # remove redundant/obsolete actions, this will remove the last two actions:
    # we don't really need to create and immediately remove the filesystem
    b.devicetree.actions.prune()

    # print all currently scheduled actions
    for action in b.devicetree.actions.find():
        print(action)

finally:
    b.devicetree.teardown_disk_images()
    os.unlink(disk1_file)



# populator/helpers/btrfs.py
# BTTRFS backend code for populating a DeviceTree.
#
# Copyright (C) 2009-2015  Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# the GNU Lesser General Public License v.2, or (at your option) any later
# version. This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY expressed or implied, including the implied
# warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
# the GNU Lesser General Public License for more details.  You should have
# received a copy of the GNU Lesser General Public License along with this
# program; if not, write to the Free Software Foundation, Inc., 51 Franklin
# Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat trademarks
# that are incorporated in the source code or documentation are not subject
# to the GNU Lesser General Public License and may only be used or
# replicated with the express permission of Red Hat, Inc.
#
# Red Hat Author(s): David Lehman <dlehman@redhat.com>
#

from ... import formats
from ... import udev
from ...devices import BTRFSVolumeDevice, BTRFSSnapShotDevice, BTRFSSubVolumeDevice
from ...errors import DeviceTreeError
from .formatpopulator import FormatPopulator

import logging
log = logging.getLogger("blivet")


class BTRFSFormatPopulator(FormatPopulator):
    priority = 100
    _type_specifier = "btrfs"

    def _get_kwargs(self):
        kwargs = super(BTRFSFormatPopulator, self)._get_kwargs()
        # the format's uuid attr will contain the UUID_SUB, while the
        # overarching volume UUID will be stored as vol_uuid
        kwargs["uuid"] = self.data["ID_FS_UUID_SUB"]
        kwargs["vol_uuid"] = udev.device_get_uuid(self.data)
        return kwargs

    def run(self):
        super(BTRFSFormatPopulator, self).run()
        uuid = udev.device_get_uuid(self.data)

        btrfs_dev = None
        for d in self._devicetree.devices:
            if isinstance(d, BTRFSVolumeDevice) and d.uuid == uuid:
                btrfs_dev = d
                break

        if btrfs_dev:
            log.info("found btrfs volume %s", btrfs_dev.name)
            btrfs_dev.parents.append(self.device)
        else:
            label = udev.device_get_label(self.data)
            log.info("creating btrfs volume btrfs.%s", label)
            btrfs_dev = BTRFSVolumeDevice(label, parents=[self.device], uuid=uuid,
                                          exists=True)
            self._devicetree._add_device(btrfs_dev)

        if not btrfs_dev.subvolumes:
            snapshots = btrfs_dev.list_subvolumes(snapshots_only=True)
            snapshot_ids = [s.id for s in snapshots]
            for subvol_dict in btrfs_dev.list_subvolumes():
                vol_id = subvol_dict.id
                vol_path = subvol_dict.path
                parent_id = subvol_dict.parent_id
                if vol_path in [sv.name for sv in btrfs_dev.subvolumes]:
                    continue

                # look up the parent subvol
                parent = None
                subvols = [btrfs_dev] + btrfs_dev.subvolumes
                for sv in subvols:
                    if sv.vol_id == parent_id:
                        parent = sv
                        break

                if parent is None:
                    log.error("failed to find parent (%d) for subvol %s",
                              parent_id, vol_path)
                    raise DeviceTreeError("could not find parent for subvol")

                fmt = formats.get_format("btrfs",
                                         device=btrfs_dev.path,
                                         exists=True,
                                         vol_uuid=btrfs_dev.format.vol_uuid,
                                         subvolspec=vol_path,
                                         mountopts="subvol=%s" % vol_path)
                if vol_id in snapshot_ids:
                    device_class = BTRFSSnapShotDevice
                else:
                    device_class = BTRFSSubVolumeDevice

                subvol = device_class(vol_path,
                                      vol_id=vol_id,
                                      fmt=fmt,
                                      parents=[parent],
                                      exists=True)
                self._devicetree._add_device(subvol)



import abc

from six import add_metaclass

from .. import util
from ..errors import FSWriteUUIDError

from . import availability
from . import fstask
from . import task


@add_metaclass(abc.ABCMeta)
class FSWriteUUID(task.BasicApplication, fstask.FSTask):

    """ An abstract class that represents writing an UUID for a filesystem. """

    description = "write filesystem UUID"

    args = abc.abstractproperty(doc="arguments for writing a UUID")

    # IMPLEMENTATION methods

    @property
    def _set_command(self):
        """Get the command to set UUID of the filesystem.

           :return: the command
           :rtype: list of str
        """
        return [str(self.ext)] + self.args

    def do_task(self):  # pylint: disable=arguments-differ
        error_msgs = self.availability_errors
        if error_msgs:
            raise FSWriteUUIDError("\n".join(error_msgs))

        rc = util.run_program(self._set_command)
        if rc:
            msg = "setting UUID via {} failed".format(self._set_command)
            raise FSWriteUUIDError(msg)


class Ext2FSWriteUUID(FSWriteUUID):
    ext = availability.TUNE2FS_APP

    @property
    def args(self):
        return ["-U", self.fs.uuid, self.fs.device]


class JFSWriteUUID(FSWriteUUID):
    ext = availability.JFSTUNE_APP

    @property
    def args(self):
        return ["-U", self.fs.uuid, self.fs.device]


class NTFSWriteUUID(FSWriteUUID):
    ext = availability.NTFSLABEL_APP

    @property
    def args(self):
        return ["--new-serial=" + self.fs.uuid, self.fs.device]


class ReiserFSWriteUUID(FSWriteUUID):
    ext = availability.REISERFSTUNE_APP

    @property
    def args(self):
        return ["-u", self.fs.uuid, self.fs.device]


class XFSWriteUUID(FSWriteUUID):
    ext = availability.XFSADMIN_APP

    @property
    def args(self):
        return ["-U", self.fs.uuid, self.fs.device]


class UnimplementedFSWriteUUID(fstask.UnimplementedFSTask):
    pass



#
# Copyright (C) 2014  Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# the GNU General Public License v.2, or (at your option) any later version.
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY expressed or implied, including the implied warranties of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.  You should have received a copy of the
# GNU General Public License along with this program; if not, write to the
# Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.  Any Red Hat trademarks that are incorporated in the
# source code or documentation are not subject to the GNU General Public
# License and may only be used or replicated with the express permission of
# Red Hat, Inc.
#
# Red Hat Author(s): Vratislav Podzimek <vpodzime@redhat.com>
#

"""
Module providing classes defining the callbacks used by Blivet and their
arguments.

"""

from collections import namedtuple

# A private namedtuple class with self-descriptive fields for passing callbacks
# to the blivet.do_it method. Each field should be populated with a function
# taking the matching CallbackTypeData (create_format_pre ->
# CreateFormatPreData, etc.)  object or None if no such callback is provided.
_CallbacksRegister = namedtuple("_CallbacksRegister",
                                ["create_format_pre",
                                 "create_format_post",
                                 "resize_format_pre",
                                 "resize_format_post",
                                 "wait_for_entropy",
                                 "report_progress"])


def create_new_callbacks_register(create_format_pre=None,
                                  create_format_post=None,
                                  resize_format_pre=None,
                                  resize_format_post=None,
                                  wait_for_entropy=None,
                                  report_progress=None):
    """
    A function for creating a new opaque object holding the references to
    callbacks. The point of this function is to hide the implementation of such
    object and to provide default values for non-specified fields (e.g. newly
    added callbacks).

    :type create_format_pre: :class:`.CreateFormatPreData` -> NoneType
    :type create_format_post: :class:`.CreateFormatPostData` -> NoneType
    :type resize_format_pre: :class:`.ResizeFormatPreData` -> NoneType
    :type resize_format_post: :class:`.ResizeFormatPostData` -> NoneType
    :param wait_for_entropy: callback for waiting for enough entropy whose return
                             value indicates whether continuing regardless of
                             available entropy should be forced (True) or not (False)
    :type wait_for_entropy: :class:`.WaitForEntropyData` -> bool
    :type report_progress: :class:`.ReportProgressData` -> NoneType

    """

    return _CallbacksRegister(create_format_pre, create_format_post,
                              resize_format_pre, resize_format_post,
                              wait_for_entropy, report_progress)


CreateFormatPreData = namedtuple("CreateFormatPreData",
                                 ["msg"])
CreateFormatPostData = namedtuple("CreateFormatPostData",
                                  ["msg"])
ResizeFormatPreData = namedtuple("ResizeFormatPreData",
                                 ["msg"])
ResizeFormatPostData = namedtuple("ResizeFormatPostData",
                                  ["msg"])
WaitForEntropyData = namedtuple("WaitForEntropyData",
                                ["msg", "min_entropy"])
ReportProgressData = namedtuple("ReportProgressData",
                                ["msg"])


#
# Callbacks for changes to the model.
#
class CallbackList(object):
    def __init__(self):
        self._cb_list = list()

    def add(self, cb):
        """ Add a callback. """
        self._cb_list.append(cb)

    def remove(self, cb):
        """ Remove a callback. """
        self._cb_list.remove(cb)

    def __call__(self, *args, **kwargs):
        """ Run all of the callbacks. """
        for cb in self._cb_list:
            cb(*args, **kwargs)


class Callbacks(object):
    """A collection of callbacks for various events

       Each trigger/event gets a list of callbacks to run, represented by an
       instance of :class:`~.callbacks.CallbackList`.
    """
    def __init__(self):
        self.populate_started = CallbackList()
        """callback list for when devicetree population is started"""

        self.device_scanned = CallbackList()
        """callback list for when a device scan is completed"""

        self.device_added = CallbackList()
        """callback list for when a device is added to the devicetree"""

        self.device_removed = CallbackList()
        """callback list for when a device is removed from the devicetree"""

        self.format_added = CallbackList()
        """callback list for when a format is added to a device"""

        self.format_removed = CallbackList()
        """callback list for when a format is removed from a device"""

        self.action_added = CallbackList()
        """ callback list for when an action is added/registered/scheduled"""

        self.action_removed = CallbackList()
        """ callback list for when an action is removed/canceled"""

        self.action_executed = CallbackList()
        """ callback list for when an action is executed"""

        self.parent_added = CallbackList()
        """ callback list for when a member device is added to a container device"""

        self.parent_removed = CallbackList()
        """ callback list for when a member device is removed from a container device"""

        self.attribute_changed = CallbackList()
        """ callback list for when a device or format attribute's value is changed"""


"""
    .. data:: callbacks

        .. note::
           The arguments for these callbacks are provided by name, so any callbacks
           you provide should be able to handle that.

        .. function:: populate_started_cb(n_devices)

           Devicetree population was started.

           :param int n_devices: (expected) total number of devices to scan


        .. function:: device_scanned_cb(device_name)

           A device scan was finished (note that some devices may be scanned
           multiple times).

           :param str device_name: name of the device that was scanned


        .. function:: device_added_cb(device)

           A device was added to the devicetree.

           :param device: the device that was added
           :type device: :class:`~.devices.StorageDevice`


        .. function:: device_removed_cb(device)

           A device was removed from the devicetree.

           :param device: the device instance that was removed
           :type device: :class:`~.devices.StorageDevice`


        .. function:: format_added_cb(device, fmt)

           A new format was added to the device.

           :param device: the device
           :type device: :class:`~.devices.StorageDevice`
           :param fmt: the added format
           :type fmt: :class:`~.formats.DeviceFormat`


        .. function:: format_removed_cb(device, fmt)

           A format was removed from the device.

           :param device: the device
           :type device: :class:`~.devices.StorageDevice`
           :param fmt: the removed format
           :type fmt: :class:`~.formats.DeviceFormat`


        .. function:: action_added_cb(action)

           An action was scheduled/registered/added.

           :param action: the action
           :type action: :class:`~.deviceaction.DeviceAction`


        .. function:: action_removed_cb(action)

           An action was canceled/removed.

           :param action: the action
           :type action: :class:`~.deviceaction.DeviceAction`


        .. function:: action_executed_cb(action)

           An action was executed/completed.

           :param action: the action
           :type action: :class:`~.deviceaction.DeviceAction`


        .. function:: parent_added_cb(device, parent)

           A member device was added to a container device.

           :param device: the member device
           :type device: :class:`~.devices.StorageDevice`
           :param device: the container device
           :type device: :class:`~.devices.StorageDevice`


        .. function:: parent_removed_cb(device, parent)

           A member device was removed from a container device.

           :param device: the device instance that was removed
           :type device: :class:`~.devices.StorageDevice`
           :param device: the container device
           :type device: :class:`~.devices.StorageDevice`


        .. function:: attribute_changed_cb(device, attr, old, new, fmt=None)

           An attribute value was changed.

           :param device: the device
           :type device: :class:`~.devices.StorageDevice`
           :param str attr: the attribute name
           :param old: the old value
           :param new: the new value
           :keyword fmt: the format, if a format attribute is what changed
           :type fmt: :class:`~.formats.DeviceFormat`


"""
callbacks = Callbacks()




import abc
import six

from . import loopbackedtestcase
from blivet.errors import FSError, FSReadLabelError
from blivet.size import Size


@six.add_metaclass(abc.ABCMeta)
class LabelingAsRoot(loopbackedtestcase.LoopBackedTestCase):

    """Tests various aspects of labeling a filesystem where there
       is no easy way to read the filesystem's label once it has been
       set and where the filesystem can not be relabeled.
    """

    _fs_class = abc.abstractproperty(
        doc="The class of the filesystem being tested on.")

    _invalid_label = abc.abstractproperty(
        doc="A label which is invalid for this filesystem.")

    _DEVICE_SIZE = Size("100 MiB")

    def __init__(self, methodName='run_test'):
        super(LabelingAsRoot, self).__init__(methodName=methodName, device_spec=[self._DEVICE_SIZE])

    def setUp(self):
        an_fs = self._fs_class()
        if not an_fs.formattable:
            self.skipTest("can not create filesystem %s" % an_fs.name)
        if not an_fs.labeling():
            self.skipTest("can not label filesystem %s" % an_fs.name)
        super(LabelingAsRoot, self).setUp()

    def test_labeling(self):
        """A sequence of tests of filesystem labeling.

           * create the filesystem when passing an invalid label
           * raise an exception when reading the filesystem
           * raise an exception when relabeling the filesystem
        """
        an_fs = self._fs_class(device=self.loop_devices[0], label=self._invalid_label)
        if an_fs._readlabel.availability_errors or not an_fs.relabels():
            self.skipTest("can not read or write label for filesystem %s" % an_fs.name)
        self.assertIsNone(an_fs.create())

        with self.assertRaises(FSReadLabelError):
            an_fs.read_label()

        an_fs.label = "an fs"
        with self.assertRaises(FSError):
            an_fs.write_label()

    def test_creating(self):
        """Create the filesystem when passing a valid label """
        an_fs = self._fs_class(device=self.loop_devices[0], label="start")
        self.assertIsNone(an_fs.create())

    def test_creating_none(self):
        """Create the filesystem when passing None
           (indicates filesystem default)
        """
        an_fs = self._fs_class(device=self.loop_devices[0], label=None)
        self.assertIsNone(an_fs.create())

    def test_creating_empty(self):
        """Create the filesystem when passing the empty label."""
        an_fs = self._fs_class(device=self.loop_devices[0], label="")
        self.assertIsNone(an_fs.create())


class LabelingWithRelabeling(LabelingAsRoot):

    """Tests labeling where it is possible to relabel.
    """

    def test_labeling(self):
        """A sequence of tests of filesystem labeling.

           * create the filesystem when passing an invalid label
           * raise an exception when reading the filesystem
           * relabel the filesystem with a valid label
           * relabel the filesystem with an empty label
           * raise an exception when relabeling when None is specified
           * raise an exception when relabeling with an invalid label
        """
        an_fs = self._fs_class(device=self.loop_devices[0], label=self._invalid_label)
        if an_fs._readlabel.availability_errors or not an_fs.relabels():
            self.skipTest("can not read or write label for filesystem %s" % an_fs.name)
        self.assertIsNone(an_fs.create())

        with self.assertRaises(FSReadLabelError):
            an_fs.read_label()

        an_fs.label = "an fs"
        self.assertIsNone(an_fs.write_label())

        an_fs.label = ""
        self.assertIsNone(an_fs.write_label())

        an_fs.label = None
        with six.assertRaisesRegex(self, FSError, "default label"):
            an_fs.write_label()

        an_fs.label = self._invalid_label
        with six.assertRaisesRegex(self, FSError, "bad label format"):
            an_fs.write_label()


class CompleteLabelingAsRoot(LabelingAsRoot):

    """Tests where it is possible to read the label and to relabel
       an existing filesystem.
    """

    def test_labeling(self):
        """A sequence of tests of filesystem labeling.

           * create the filesystem when passing an invalid label
             and verify that the filesystem has the default label
           * relabel the filesystem with a valid label
             and verify that the filesystem has that label
           * relabel the filesystem with an empty label
             and verify that the filesystem has that label
           * raise an exception when relabeling when None is specified
           * raise an exception when relabeling with an invalid label
        """
        an_fs = self._fs_class(device=self.loop_devices[0], label=self._invalid_label)
        if an_fs._readlabel.availability_errors or not an_fs.relabels():
            self.skipTest("can not read or write label for filesystem %s" % an_fs.name)
        self.assertIsNone(an_fs.create())
        self.assertEqual(an_fs.read_label(), an_fs._labelfs.default_label)

        an_fs.label = "an_fs"
        self.assertIsNone(an_fs.write_label())
        self.assertEqual(an_fs.read_label(), an_fs.label)

        an_fs.label = ""
        self.assertIsNone(an_fs.write_label())
        self.assertEqual(an_fs.read_label(), an_fs.label)

        an_fs.label = None
        with six.assertRaisesRegex(self, FSError, "default label"):
            an_fs.write_label()

        an_fs.label = "n" * 129
        with six.assertRaisesRegex(self, FSError, "bad label format"):
            an_fs.write_label()

    def test_creating(self):
        """Create the filesystem when passing a valid label.
           Verify that the filesystem has that label.
        """
        an_fs = self._fs_class(device=self.loop_devices[0], label="start")
        if an_fs._readlabel.availability_errors:
            self.skipTest("can not read label for filesystem %s" % an_fs.name)
        self.assertIsNone(an_fs.create())
        self.assertEqual(an_fs.read_label(), "start")

    def test_creating_none(self):
        """Create a filesystem with the label None.
           Verify that the filesystem has the default label.
        """
        an_fs = self._fs_class(device=self.loop_devices[0], label=None)
        if an_fs._readlabel.availability_errors:
            self.skipTest("can not read label for filesystem %s" % an_fs.name)
        self.assertIsNone(an_fs.create())
        self.assertEqual(an_fs.read_label(), an_fs._labelfs.default_label)

    def test_creating_empty(self):
        """Create a filesystem with an empty label.
           Verify that the filesystem has the empty label.
        """
        an_fs = self._fs_class(device=self.loop_devices[0], label="")
        if an_fs._readlabel.availability_errors:
            self.skipTest("can not read label for filesystem %s" % an_fs.name)
        self.assertIsNone(an_fs.create())
        self.assertEqual(an_fs.read_label(), "")



#!/usr/bin/python3

import sys
from pocketlint import PocketLintConfig, PocketLinter

class TranslationCanaryLintConfig(PocketLintConfig):
    @property
    def disabledOptions(self):
        return [ "I0011",           # Locally disabling %s
               ]

    @property
    def extraArgs(self):
        return ["--init-import", "y"]

if __name__ == "__main__":
    conf = TranslationCanaryLintConfig()
    linter = PocketLinter(conf)
    rc = linter.run()
    sys.exit(rc)



# populator/helpers/partition.py
# Partition backend code for populating a DeviceTree.
#
# Copyright (C) 2009-2015  Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# the GNU Lesser General Public License v.2, or (at your option) any later
# version. This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY expressed or implied, including the implied
# warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
# the GNU Lesser General Public License for more details.  You should have
# received a copy of the GNU Lesser General Public License along with this
# program; if not, write to the Free Software Foundation, Inc., 51 Franklin
# Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat trademarks
# that are incorporated in the source code or documentation are not subject
# to the GNU Lesser General Public License and may only be used or
# replicated with the express permission of Red Hat, Inc.
#
# Red Hat Author(s): David Lehman <dlehman@redhat.com>
#

import copy
import six

from ... import udev
from ...devices import PartitionDevice
from ...errors import DeviceError
from ...formats import get_format
from ...storage_log import log_method_call
from .devicepopulator import DevicePopulator

import logging
log = logging.getLogger("blivet")


class PartitionDevicePopulator(DevicePopulator):
    priority = 0

    @classmethod
    def match(cls, data):
        return (udev.device_is_partition(data) or udev.device_is_dm_partition(data))

    def run(self):
        name = udev.device_get_name(self.data)
        log_method_call(self, name=name)
        sysfs_path = udev.device_get_sysfs_path(self.data)

        device = self._devicetree.get_device_by_name(name)
        if device:
            return device

        disk = None
        disk_name = udev.device_get_partition_disk(self.data)
        if disk_name:
            disk = self._devicetree.get_device_by_name(disk_name)
            if disk is None:
                # create a device instance for the disk
                disk_info = six.next((i for i in udev.get_devices()
                                      if udev.device_get_name(i) == disk_name), None)
                if disk_info is not None:
                    self._devicetree.handle_device(disk_info)
                    disk = self._devicetree.get_device_by_name(disk_name)

        if disk is None:
            # if the disk is still not in the tree something has gone wrong
            log.error("failure finding disk for %s", name)
            return

        if not disk.partitioned or not disk.format.supported:
            # Ignore partitions on:
            #  - devices we do not support partitioning of, like logical volumes
            #  - devices that contain disklabels made by isohybrid
            #
            # For partitions on disklabels parted cannot make sense of, go ahead
            # and instantiate a PartitionDevice so our view of the layout is
            # complete.
            if not disk.partitionable or disk.format.type in ("iso9660", "udf") or disk.format.hidden:
                log.debug("ignoring partition %s on %s", name, disk.format.type)
                return

            if not disk.partitioned:
                log.info("ignoring '%s' format on disk that contains '%s'", disk.format.type, name)
                disk.format = get_format("disklabel", exists=True, device=disk.path)
                disk.original_format = copy.deepcopy(disk.format)

        try:
            device = PartitionDevice(name, sysfs_path=sysfs_path,
                                     uuid=udev.device_get_partition_uuid(self.data),
                                     major=udev.device_get_major(self.data),
                                     minor=udev.device_get_minor(self.data),
                                     exists=True, parents=[disk])
        except DeviceError as e:
            # This should only happen for the magic whole-disk partition on a sun disklabel.
            log.error("Failed to instantiate PartitionDevice: %s", e)
            return

        self._devicetree._add_device(device)
        return device



import time

import blivet
from blivet.events.manager import event_manager
from blivet.util import set_up_logging


def print_changes(event, changes):
    print("***", event)
    for change in changes:
        print("***", change)
    print("***")
    print()


set_up_logging(console_logs=["blivet.event"])
b = blivet.Blivet()  # create an instance of Blivet
b.reset()  # detect system storage configuration
print(b.devicetree)

event_manager.notify_cb = print_changes
event_manager.enable()

while True:
    time.sleep(0.5)



# size.py
# Python module to represent storage sizes
#
# Copyright (C) 2010  Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# the GNU General Public License v.2, or (at your option) any later version.
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY expressed or implied, including the implied warranties of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.  You should have received a copy of the
# GNU General Public License along with this program; if not, write to the
# Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.  Any Red Hat trademarks that are incorporated in the
# source code or documentation are not subject to the GNU General Public
# License and may only be used or replicated with the express permission of
# Red Hat, Inc.
#
# Red Hat Author(s): David Cantrell <dcantrell@redhat.com>

from bytesize import bytesize

# we just need to make these objects available here
# pylint: disable=unused-import
from bytesize.bytesize import B, KiB, MiB, GiB, TiB, PiB, EiB, ZiB, YiB, KB, MB, GB, TB, PB, EB, ZB, YB
from bytesize.bytesize import ROUND_UP, ROUND_DOWN, ROUND_HALF_UP


def unit_str(unit, xlate=False):
    """ Return a string representation of unit.

        :param unit: a named unit, e.g., KiB
        :param bool xlate: if True, translate to current locale
        :rtype: some kind of string type
        :returns: string representation of unit
    """
    return bytesize.unit_str(unit, xlate)


class Size(bytesize.Size):
    """ Common class to represent storage device and filesystem sizes.
        Can handle parsing strings such as 45MB or 6.7GB to initialize
        itself, or can be initialized with a numerical size in bytes.
        Also generates human readable strings to a specified number of
        decimal places.
    """

    def __abs__(self):
        return Size(bytesize.Size.__abs__(self))

    def __add__(self, other):
        return Size(bytesize.Size.__add__(self, other))

    # needed to make sum() work with Size arguments
    def __radd__(self, other):
        return Size(bytesize.Size.__radd__(self, other))

    def __sub__(self, other):
        return Size(bytesize.Size.__sub__(self, other))

    def __rsub__(self, other):
        return Size(bytesize.Size.__rsub__(self, other))

    def __mul__(self, other):
        return Size(bytesize.Size.__mul__(self, other))
    __rmul__ = __mul__

    def __div__(self, other):       # pylint: disable=unused-argument
        ret = bytesize.Size.__div__(self, other)  # pylint: disable=no-member
        if isinstance(ret, bytesize.Size):
            ret = Size(ret)

        return ret

    def __truediv__(self, other):
        ret = bytesize.Size.__truediv__(self, other)
        if isinstance(ret, bytesize.Size):
            ret = Size(ret)

        return ret

    def __floordiv__(self, other):
        ret = bytesize.Size.__floordiv__(self, other)
        if isinstance(ret, bytesize.Size):
            ret = Size(ret)

        return ret

    def __mod__(self, other):
        return Size(bytesize.Size.__mod__(self, other))

    def __deepcopy__(self, memo_dict):
        return Size(bytesize.Size.__deepcopy__(self, memo_dict))

    # pylint: disable=arguments-differ,arguments-renamed
    def convert_to(self, spec=None):
        """ Return the size in the units indicated by the specifier.

            :param spec: a units specifier
            :type spec: a units specifier or :class:`Size`
            :returns: a numeric value in the units indicated by the specifier
            :rtype: Decimal
            :raises ValueError: if Size unit specifier is non-positive

            .. versionadded:: 1.6
               spec parameter may be Size as well as units specifier.
        """
        if isinstance(spec, Size):
            if spec == Size(0):
                raise ValueError("cannot convert to 0 size")
            return bytesize.Size.__truediv__(self, spec)
        spec = B if spec is None else spec
        return bytesize.Size.convert_to(self, spec)

    def human_readable(self, min_unit=B, max_places=2, xlate=True):
        """ Return a string representation of this size with appropriate
            size specifier and in the specified number of decimal places.
            Values are always represented using binary not decimal units.
            For example, if the number of bytes represented by this size
            is 65531, expect the representation to be something like
            64.00 KiB, not 65.53 KB.

            :param min_unit: the smallest unit the returned representation should use
            :type min_unit: one of the B, KiB, MiB,... (binary) units from this module
                            or str ("B", "KiB",...)
            :param max_places: number of decimal places to use
            :type max_places: an integer type or NoneType
            :param bool xlate: If True, translate for current locale
            :returns: a representation of the size
            :rtype: str

        """

        if max_places is None:
            max_places = -1
        return bytesize.Size.human_readable(self, min_unit, max_places, xlate)

    # pylint: disable=arguments-differ,arguments-renamed
    def round_to_nearest(self, size, rounding):
        """ Rounds to nearest unit specified as a named constant or a Size.

            :param size: a size specifier
            :type size: a named constant like KiB, or any non-negative Size
            :keyword rounding: which direction to round
            :type rounding: one of ROUND_UP, ROUND_DOWN, or ROUND_HALF_UP
            :returns: Size rounded to nearest whole specified unit
            :rtype: :class:`Size`

            If size is Size(0), returns Size(0).
        """
        if rounding not in (ROUND_UP, ROUND_DOWN, ROUND_HALF_UP):
            raise ValueError("invalid rounding specifier")

        if isinstance(size, Size):
            if size.get_bytes() == 0:
                return Size(0)
            elif size < Size(0):
                raise ValueError("invalid rounding size: %s" % size)

        return Size(bytesize.Size.round_to_nearest(self, size, rounding))

    def ensure_percent_reserve(self, percent):
        """Get a new size with given space reserve.

            :param percent: number of percent to reserve
            :returns: a new size with :param:`percent` space reserve
            :rtype: :class:`Size`

            Best described with an example:
            >>> Size("80 GiB").ensure_percent_reserve(20)
            Size (100 GiB)

            This method returns a new size in which there's a :param:`percent`%
            reserve (on top of the original size). In other words::

              with_reserve - orig_size == with_reserve * (percent / 100)

            except that due to float arithmetics the numbers are unlikely to be
            exactly equal.
        """
        return Size(self * (1 / (1 - (percent / 100))))



from .devicelibs_test import *
from .devices_test import *
from .formats_tests import *

from .action_test import *
from .blivettestcase import *
from .dbus_test import *
from .devicefactory_test import *
from .devicetree_test import *
from .events_test import *
from .misc_test import *
from .parentlist_test import *
from .populator_test import *
from .size_test import *
from .task_tests import *
from .tsort_test import *
from .udev_test import *
from .util_test import *



# gpt_test.py
# GPT partitioning helpers test suite
#
# Copyright (C) Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# the GNU General Public License v.2, or (at your option) any later version.
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY expressed or implied, including the implied warranties of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.  You should have received a copy of the
# GNU General Public License along with this program; if not, write to the
# Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.  Any Red Hat trademarks that are incorporated in the
# source code or documentation are not subject to the GNU General Public
# License and may only be used or replicated with the express permission of
# Red Hat, Inc.
#

import unittest
from uuid import UUID

try:
    from unittest import mock
except ImportError:
    import mock

from blivet.devicelibs import gpt
from blivet.errors import GPTVolUUIDError


class GPTTestCase(unittest.TestCase):

    def test_uuid_for_vol_arch_less(self):
        uuid = gpt.gpt_part_uuid_for_volume(gpt.GPT_VOL_SRV)
        self.assertEqual(uuid, UUID("3b8f8425-20e0-4f3b-907f-1a25a76f98e8"))

    @mock.patch("blivet.devicelibs.gpt.get_arch", return_value="ppc64")
    def test_uuid_for_vol_arch_implicit(self, mock_get_arch):
        uuid = gpt.gpt_part_uuid_for_volume(gpt.GPT_VOL_ARCH_USR_VERITY)
        mock_get_arch.assert_called_once()
        self.assertEqual(uuid, UUID("bdb528a5-a259-475f-a87d-da53fa736a07"))

    def test_uuid_for_vol_arch_explicit(self):
        uuid = gpt.gpt_part_uuid_for_volume(gpt.GPT_VOL_ARCH_USR_VERITY,
                                            arch="s390")
        self.assertEqual(uuid, UUID("b663c618-e7bc-4d6d-90aa-11b756bb1797"))

    def test_uuid_for_vol_arch_unknown(self):
        with self.assertRaises(GPTVolUUIDError):
            gpt.gpt_part_uuid_for_volume(gpt.GPT_VOL_ARCH_USR_VERITY,
                                         arch="nope")

    def test_uuid_for_vol_unknown(self):
        with self.assertRaises(GPTVolUUIDError):
            gpt.gpt_part_uuid_for_volume("nope")

    def test_uuid_for_path_arch_less(self):
        uuid = gpt.gpt_part_uuid_for_mountpoint("/home")
        self.assertEqual(uuid, UUID("933ac7e1-2eb4-4f13-b844-0e14e2aef915"))

    @mock.patch("blivet.devicelibs.gpt.get_arch", return_value="ppc64")
    def test_uuid_for_path_arch_implicit(self, mock_get_arch):
        uuid = gpt.gpt_part_uuid_for_mountpoint("/")
        mock_get_arch.assert_called_once()
        self.assertEqual(uuid, UUID("912ade1d-a839-4913-8964-a10eee08fbd2"))

    def test_uuid_for_path_arch_explicit(self):
        uuid = gpt.gpt_part_uuid_for_mountpoint("/",
                                                arch="s390")
        self.assertEqual(uuid, UUID("08a7acea-624c-4a20-91e8-6e0fa67d23f9"))

    def test_uuid_for_path_unknown(self):
        uuid = gpt.gpt_part_uuid_for_mountpoint("/nope")
        self.assertEqual(uuid, None)
